/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 1.6.0, packaged on August 2008.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "AlbumFile.h"
#include <GLC_FileFormatException>
#include <GLC_Vector4d>
#include <GLC_Exception>

AlbumFile::AlbumFile()
: m_pStreamWriter(NULL)
, m_pStreamReader(NULL)
, m_SessionPathDir()
{
}

AlbumFile::~AlbumFile()
{
	if (NULL != m_pStreamWriter)
	{
		delete m_pStreamWriter;
	}
	if (NULL != m_pStreamReader)
	{
		delete m_pStreamReader;
	}

}

/////////////////////////////////////////////////////////////////////
// Public Interface
//////////////////////////////////////////////////////////////////////
// Load a album file
/* Return the file entry list as result
 */
QList<FileEntry> AlbumFile::loadAlbumFile(QFile* pSessionFile)
{
	Q_ASSERT(NULL == m_pStreamReader);
	
	// Open the session file in read only mode
	pSessionFile->open(QIODevice::ReadOnly);
	// The Session Path Dir
	m_SessionPathDir= QFileInfo(*pSessionFile).absoluteDir();

	m_pStreamReader= new QXmlStreamReader(pSessionFile);
	// Check the session file validity
	if (!checkAlbumFileValidity())
	{
		QString message("Album file not valid!");
		GLC_Exception e(message);
		delete m_pStreamReader;
		m_pStreamReader= NULL;
		throw e;
	}
	// The list of FileEntry to return
	QList<FileEntry> returnEntries;
	while (!m_pStreamReader->atEnd())
	{
		if ((QXmlStreamReader::StartElement == m_pStreamReader->readNext()) && (m_pStreamReader->name() == "Model"))
		{
			// Find Model element going to parse it
			returnEntries.append(currentModelToFileEnty());
		}
	}
	
	// Close the session file
	pSessionFile->close();
	
	delete m_pStreamReader;
	m_pStreamReader= NULL;
	
	return returnEntries;
}

// Save a session file
void AlbumFile::saveAlbumFile(const QList<FileEntry>& fileEntryList, QFile* pSessionFile)
{
	Q_ASSERT(NULL == m_pStreamWriter);
	// Open the Session file in write only mode
	pSessionFile->open(QIODevice::WriteOnly);
	// The Session Path Dir
	m_SessionPathDir= QFileInfo(*pSessionFile).absoluteDir();
	
	m_pStreamWriter= new QXmlStreamWriter(pSessionFile);
	m_pStreamWriter->setAutoFormatting(true);
	// Begin to write the xml document
	m_pStreamWriter->writeStartDocument();
	m_pStreamWriter->writeStartElement("Album");
	writeHeader();
	m_pStreamWriter->writeStartElement("Root");
	
	// File Entry list processing
	const int max= fileEntryList.size();
	for (int i = 0; i < max; ++i)
	{
		writeModel(fileEntryList[i]);
	}
	
	// Finish the writing of document
	m_pStreamWriter->writeEndElement(); // Root
	m_pStreamWriter->writeEndElement(); // Session
	m_pStreamWriter->writeEndDocument();
	
	// Close the session file
	pSessionFile->close();
	
	delete m_pStreamWriter;
	m_pStreamWriter= NULL;
}

//////////////////////////////////////////////////////////////////////
// private member
//////////////////////////////////////////////////////////////////////
//! Write the header of xml document
void AlbumFile::writeHeader() const
{
	m_pStreamWriter->writeStartElement("Header");
	m_pStreamWriter->writeStartElement("Application");
	m_pStreamWriter->writeAttribute("Name", QCoreApplication::applicationName());
	m_pStreamWriter->writeAttribute("Version", "1.5");
	m_pStreamWriter->writeEndElement(); // Application
	m_pStreamWriter->writeEndElement(); // Header
}
//! Write a model in the current xml document
void AlbumFile::writeModel(const FileEntry& fileEntry)
{	
	m_pStreamWriter->writeStartElement("Model");
	m_pStreamWriter->writeAttribute("AFileName", fileEntry.getFileName());
	m_pStreamWriter->writeAttribute("RFileName", m_SessionPathDir.relativeFilePath(fileEntry.getFileName()));
	
	m_pStreamWriter->writeStartElement("Camera");
	double angle= fileEntry.getViewAngle();
	m_pStreamWriter->writeAttribute("Angle", QString::number(angle));
	
	m_pStreamWriter->writeStartElement("Eye");
	double x= fileEntry.getCamera().getEye().getX();
	m_pStreamWriter->writeAttribute("x", QString::number(x));
	double y= fileEntry.getCamera().getEye().getY();
	m_pStreamWriter->writeAttribute("y", QString::number(y));
	double z= fileEntry.getCamera().getEye().getZ();
	m_pStreamWriter->writeAttribute("z", QString::number(z));
	m_pStreamWriter->writeEndElement(); // Eye

	m_pStreamWriter->writeStartElement("Target");
	x= fileEntry.getCamera().getTarget().getX();
	m_pStreamWriter->writeAttribute("x", QString::number(x));
	y= fileEntry.getCamera().getTarget().getY();
	m_pStreamWriter->writeAttribute("y", QString::number(y));
	z= fileEntry.getCamera().getTarget().getZ();
	m_pStreamWriter->writeAttribute("z", QString::number(z));
	m_pStreamWriter->writeEndElement(); // Target

	m_pStreamWriter->writeStartElement("Up");
	x= fileEntry.getCamera().getVectUp().getX();
	m_pStreamWriter->writeAttribute("x", QString::number(x));
	y= fileEntry.getCamera().getVectUp().getY();
	m_pStreamWriter->writeAttribute("y", QString::number(y));
	z= fileEntry.getCamera().getVectUp().getZ();
	m_pStreamWriter->writeAttribute("z", QString::number(z));
	m_pStreamWriter->writeEndElement(); // Up
	
	m_pStreamWriter->writeEndElement(); // Camera
	m_pStreamWriter->writeEndElement(); // Model

}
// Check if the session file is valid
bool AlbumFile::checkAlbumFileValidity()
{
	// trying to find start Document of the xml file
	while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Album" && !m_pStreamReader->atEnd());
	if (m_pStreamReader->atEnd()) return false;
	if (m_pStreamReader->hasError()) return false;
	
	// Trying to find the header of the album
	while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Header" && !m_pStreamReader->atEnd());
	if (m_pStreamReader->atEnd()) return false;
	if (m_pStreamReader->hasError()) return false;
	
	// Cheking the validity of the header
	// Trying to find application element
	while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Application" && !m_pStreamReader->atEnd());
	if (m_pStreamReader->atEnd()) return false;
	if (m_pStreamReader->hasError()) return false;
	// Trying to find attribute Name
	if (m_pStreamReader->attributes().value("Name") != QCoreApplication::applicationName()) return false;
	if (m_pStreamReader->attributes().value("Version") != "1.5") return false;
	if (m_pStreamReader->hasError()) return false;
	
	//Trying to find root element
	while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Root" && !m_pStreamReader->atEnd());
	if (m_pStreamReader->atEnd()) return false;
	if (m_pStreamReader->hasError()) return false;
	
	int NumberOfModels= 0;
	// Trying to find model elements
	while(!m_pStreamReader->atEnd())
	{
		if ((QXmlStreamReader::StartElement == m_pStreamReader->readNext()) && (m_pStreamReader->name() == "Model"))
		{
			++NumberOfModels;
			if (m_pStreamReader->attributes().value("AFileName").isEmpty()) return false;
			if (m_pStreamReader->attributes().value("RFileName").isEmpty()) return false;
			//Trying to find Camera element
			while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Camera" && !m_pStreamReader->atEnd());
			if (m_pStreamReader->atEnd()) return false;
			if (m_pStreamReader->hasError()) return false;
			if (m_pStreamReader->attributes().value("Angle").isEmpty()) return false;
			//Trying to find Eye element
			while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Eye" && !m_pStreamReader->atEnd());
			if (m_pStreamReader->atEnd()) return false;
			if (m_pStreamReader->hasError()) return false;
			if (m_pStreamReader->attributes().value("x").isEmpty()) return false;
			if (m_pStreamReader->attributes().value("y").isEmpty()) return false;
			if (m_pStreamReader->attributes().value("z").isEmpty()) return false;
			//Trying to find Target element
			while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Target" && !m_pStreamReader->atEnd());
			if (m_pStreamReader->atEnd()) return false;
			if (m_pStreamReader->hasError()) return false;
			if (m_pStreamReader->attributes().value("x").isEmpty()) return false;
			if (m_pStreamReader->attributes().value("y").isEmpty()) return false;
			if (m_pStreamReader->attributes().value("z").isEmpty()) return false;
			//Trying to find Up element
			while(QXmlStreamReader::StartElement != m_pStreamReader->readNext() && m_pStreamReader->name() != "Up" && !m_pStreamReader->atEnd());
			if (m_pStreamReader->atEnd()) return false;
			if (m_pStreamReader->hasError()) return false;
			if (m_pStreamReader->attributes().value("x").isEmpty()) return false;
			if (m_pStreamReader->attributes().value("y").isEmpty()) return false;
			if (m_pStreamReader->attributes().value("z").isEmpty()) return false;			
		}
		if (m_pStreamReader->hasError()) return false;
	}
	if (0 == NumberOfModels) return false;
	
	// Reset the xml stream reader to the begining
	QIODevice* pDevice= m_pStreamReader->device();
	pDevice->reset();
	m_pStreamReader->setDevice(pDevice);
	
	return true;

}
// Convert current model xml element to a FileEntry object
FileEntry AlbumFile::currentModelToFileEnty()
{
	// The model file name
	QString fileName(m_pStreamReader->attributes().value("AFileName").toString());
	// Check if the file exists
	if (!QFile::exists(fileName))
	{
		fileName= m_pStreamReader->attributes().value("RFileName").toString();
		fileName= m_SessionPathDir.absoluteFilePath(fileName);
	}
	
	// Go to camera element
	while ((!m_pStreamReader->atEnd()) && (QXmlStreamReader::StartElement != m_pStreamReader->readNext()) && (m_pStreamReader->name() != "Camera"));
	Q_ASSERT(!m_pStreamReader->atEnd());
	// Camera Angle of view
	double angle;
	angle= m_pStreamReader->attributes().value("Angle").toString().toDouble();
	
	double x, y, z; // Tempory vector coordinate
	// Go to camera Eye vector element
	while ((!m_pStreamReader->atEnd()) && (QXmlStreamReader::StartElement != m_pStreamReader->readNext()) && (m_pStreamReader->name() != "Eye"));
	Q_ASSERT(!m_pStreamReader->atEnd());
	x= m_pStreamReader->attributes().value("x").toString().toDouble();
	y= m_pStreamReader->attributes().value("y").toString().toDouble();
	z= m_pStreamReader->attributes().value("z").toString().toDouble();
	GLC_Vector4d eye(x, y, z);

	// Go to camera Target vector element
	while ((!m_pStreamReader->atEnd()) && (QXmlStreamReader::StartElement != m_pStreamReader->readNext()) && (m_pStreamReader->name() != "Target"));
	Q_ASSERT(!m_pStreamReader->atEnd());
	x= m_pStreamReader->attributes().value("x").toString().toDouble();
	y= m_pStreamReader->attributes().value("y").toString().toDouble();
	z= m_pStreamReader->attributes().value("z").toString().toDouble();
	GLC_Vector4d target(x, y, z);

	// Go to camera Up vector element
	while ((!m_pStreamReader->atEnd()) && (QXmlStreamReader::StartElement != m_pStreamReader->readNext()) && (m_pStreamReader->name() != "Up"));
	Q_ASSERT(!m_pStreamReader->atEnd());
	x= m_pStreamReader->attributes().value("x").toString().toDouble();
	y= m_pStreamReader->attributes().value("y").toString().toDouble();
	z= m_pStreamReader->attributes().value("z").toString().toDouble();
	GLC_Vector4d up(x, y, z);
	// The model camera
	GLC_Camera camera(eye, target, up);
	
	return FileEntry(fileName, camera, angle);
}

