/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 1.6.0, packaged on August 2008.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#ifndef FILEENTRY_H_
#define FILEENTRY_H_

#include <QString>
#include <GLC_World>
#include <GLC_Camera>


//////////////////////////////////////////////////////////////////////
//! \class FileEntry
/*! \brief FileEntry : The entry of File list*/
//////////////////////////////////////////////////////////////////////
class FileEntry
{
	
//////////////////////////////////////////////////////////////////////
/*! @name Constructor / Destructor */
//@{
//////////////////////////////////////////////////////////////////////	
public:
	//! Default constructor
	FileEntry(const QString& fileName= QString());
	//! Construct from fileName and camera
	FileEntry(const QString&, const GLC_Camera&, const double&);
	//! Copy constructor
	FileEntry(const FileEntry&);
	//! the destructor
	virtual ~FileEntry();
//@}
//////////////////////////////////////////////////////////////////////
// Public Interface
//////////////////////////////////////////////////////////////////////
public:
	//! Get the File Name of this entry
	inline QString getFileName() const {return m_FileName;}
	//! Set the GLC_Instance of this file entry
	void setWorld(GLC_World&);
	//! Get the GLC_Intsance of this file entry
	inline GLC_World getWorld() const {return m_World;}
	//! return true if this file entry is loaded
	inline bool isLoaded() const {return m_IsLoaded;}
	//! return true if this file entry is loading
	inline bool isLoading() const {return m_IsLoading;}
	//! return true if this entry is ready to load
	bool isReadyToLoad() const;
	//! return true if an error occur while trying to load the file
	inline bool errorOccurWhileLoading() const {return !m_Error.isEmpty();}
	//! set the loading error
	inline void setError(const QString& error) {m_Error= error;}
	//! return the loading error
	inline QString getError() const {return m_Error;}
	//! Set the loading status of this entry
	inline void setLoadingStatus(bool status){m_IsLoading= status;}
	//! return true if the camera has been set
	bool cameraIsSet() const {return m_CameraIsSet;}
	//! Get the entry camera
	inline GLC_Camera getCamera() const {return m_Camera;}
	//! Get the Angle of view
	inline double getViewAngle() const {return m_AngleOfView;}
	//! Set the entry Camera
	inline void setCameraAndAngle(const GLC_Camera& cam, const double& angle)
	{
		m_CameraIsSet= true;
		m_Camera= cam;
		m_AngleOfView= angle;
	}
	//! Get the number of instance of this entry
	inline int getNumberOfInstance() {return m_World.collection()->getNumber();}
	//! Get the number of Faces of this entry
	inline int getNumberOfFaces() const {return m_NumberOfFaces;}
	//! Get the number of Vertex of this entry
	inline int getNumberOfVertex() const {return m_NumberOfVertex;}
	//! get the entry polygon mode
	inline GLenum getPolygonMode() const {return m_PolyMode;}
	//! Set the entry Polygon mode
	inline void setPolygonMode(GLenum mode)
	{
		m_World.collection()->setPolygonModeForAll(GL_FRONT_AND_BACK, mode);
		m_PolyMode= mode;
	}
	inline bool operator==(const FileEntry& other) const {return m_FileName == other.m_FileName;}
	
//////////////////////////////////////////////////////////////////////
// private member
//////////////////////////////////////////////////////////////////////
private:
	//! The filename associate to this entry
	QString m_FileName;
	//! this entry is in loading process
	bool m_IsLoading;
	//! The instance as been set
	bool m_IsLoaded;
	//! The GLC_World of this file entry
	GLC_World m_World;
	//! Error description if error occur while loading
	QString m_Error;
	//! The camera off entry
	GLC_Camera m_Camera;
	//! true if the camer has been set
	bool m_CameraIsSet;
	//! The polygon mode
	GLenum m_PolyMode;
	//! Number of Faces
	int m_NumberOfFaces;
	//! Number of Vertex
	int m_NumberOfVertex;
	//! the angle of view
	double m_AngleOfView;
};

#endif /*FILEENTRY_H_*/
