/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 1.6.0, packaged on August 2008.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "OpenglView.h"
#include <GLC_Instance>
#include <GLC_Interpolator>
#include <GLC_Exception>

// For VSYNC problem under Mac OS X
#if defined(Q_OS_MAC)
#include <OpenGL.h>
#endif


OpenglView::OpenglView(QWidget *pParent)
: QGLWidget(pParent)
, m_GlView(this)
, m_World()
, m_Light()
, m_Mode(GL_FILL)
, m_ViewState(V_NORMAL)
, m_ViewEnterState(VE_NORMAL)
, m_MotionStep(20)
, m_CurrentFps(0)
, m_infoFont()
, m_SelectionMode(false)
, m_dislayInfoPanel(true)
, m_SnapShootMode(false)

{
	m_Light.setPosition(1.0, 1.0, 1.0);
	
	// Set backroundImage
	m_GlView.loadBackGroundImage(":images/default_background.png");
		
}

OpenglView::~OpenglView()
{
}

//////////////////////////////////////////////////////////////////////
// Public Interface
//////////////////////////////////////////////////////////////////////

// Initialyze OpenGL window
void OpenglView::initializeGL()
{
	// Define position of the camera eye
	const GLC_Vector4d VectEye(1.0, 1.0, 1.0);
	// of the target
	const GLC_Vector4d VectTarget;
	// Up vector
	GLC_Vector4d VectUp(0.0, 1.0, 0.0);
	m_GlView.m_pViewCam->setCam(VectEye, VectTarget, VectUp);
	
	m_GlView.initGl();
	// For VSYNC problem under Mac OS X
	#if defined(Q_OS_MAC)
	const GLint swapInterval = 1;
	CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &swapInterval);
	#endif

}

// Reframe the Scene
void OpenglView::reframe(const GLC_BoundingBox& boundingBox, bool motion)
{
	const GLC_BoundingBox collectionBox= m_World.collection()->getBoundingBox();
	if (boundingBox.isEmpty())
	{		
		if (!collectionBox.isEmpty())
		{
			GLC_Camera savCam(*(m_GlView.m_pViewCam));
			m_GlView.reframe(collectionBox);
			if (motion)
			{
				GLC_Camera newCam(*(m_GlView.m_pViewCam));
				m_GlView.m_pViewCam->setCam(savCam);
				motionReframe(newCam, m_MotionStep);
			}
			else
			{
				m_GlView.setDistMinAndMax(collectionBox);
				updateGL();
				emit viewChanged();
			}
		}		
	}
	else
	{
		GLC_Camera savCam(*(m_GlView.m_pViewCam));
		m_GlView.reframe(boundingBox);
		if (savCam == *(m_GlView.m_pViewCam))
		{
			m_GlView.reframe(collectionBox);
		}
		if (motion)
		{
			GLC_Camera newCam(*(m_GlView.m_pViewCam));
			m_GlView.m_pViewCam->setCam(savCam);
			motionReframe(newCam, m_MotionStep);
		}
		else
		{
			m_GlView.setDistMinAndMax(boundingBox);
			updateGL();			
		}
	}
}

// Change The enter state of the view
void OpenglView::changeEnterState(ViewEnterState_enum state)
{
	if (m_ViewState == V_NORMAL)
	{
		m_ViewEnterState= state;
	}
}

// Set the view Camera
void OpenglView::setCameraAndAngle(const GLC_Camera& cam, const double& angle)
{
	*(m_GlView.m_pViewCam)= cam;
	m_GlView.setFov(angle);
	m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
	emit viewChanged();
}

// Iso view
void OpenglView::isoView(bool motion)
{
	const GLC_Point4d target(m_GlView.m_pViewCam->getTarget());
	
	// Define position of the camera eye
	GLC_Point4d eye(1.0, 1.0, 1.0);
	eye= eye + target;
	// Up vector
	const GLC_Vector4d vectUp(0.0, 1.0, 0.0);
	GLC_Camera newCam(eye, target, vectUp);
	newCam.setDistEyeTarget(m_GlView.m_pViewCam->getDistEyeTarget());
	if (motion)
	{
		rotateCamera(newCam, m_MotionStep);
	}
	else
	{
		m_GlView.m_pViewCam->setCam(newCam);
		m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
		updateGL();
	}
}
// Front view
void OpenglView::frontView()
{
	const GLC_Point4d target(m_GlView.m_pViewCam->getTarget());
	const double distance= m_GlView.m_pViewCam->getDistEyeTarget();
	// Define position of the camera eye
	GLC_Point4d eye(0.0, 0.0, 1.0);
	
	// Check angle
	const GLC_Vector4d vectCam((m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget()).setNormal(1));
	const double Angle= acos(vectCam * eye);
	if ((glc::PI - Angle) < glc::EPSILON)
	{
		m_GlView.m_pViewCam->setEyeCam(m_GlView.m_pViewCam->getEye() + GLC_Vector4d(0.1, 0.0, 0.0));
		m_GlView.m_pViewCam->setDistEyeTarget(distance);		
	}
	GLC_Matrix4x4 matRot(m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget(), 5 * glc::EPSILON);
	GLC_Vector4d newUp(matRot * m_GlView.m_pViewCam->getVectUp());
	m_GlView.m_pViewCam->setUpCam(newUp);
	
	eye= eye + target;
	// Up vector
	const GLC_Vector4d vectUp(0.0, 1.0, 0.0);
	GLC_Camera newCam(eye, target, vectUp);
	newCam.setDistEyeTarget(m_GlView.m_pViewCam->getDistEyeTarget());
	rotateCamera(newCam, m_MotionStep);
}
// Back view
void OpenglView::backView()
{
	const GLC_Point4d target(m_GlView.m_pViewCam->getTarget());
	const double distance= m_GlView.m_pViewCam->getDistEyeTarget();
	// Define position of the camera eye
	GLC_Point4d eye(0.0, 0.0, -1.0);
	// Check angle
	const GLC_Vector4d vectCam((m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget()).setNormal(1));
	const double Angle= acos(vectCam * eye);
	if ((glc::PI - Angle) < glc::EPSILON)
	{
		m_GlView.m_pViewCam->setEyeCam(m_GlView.m_pViewCam->getEye() + GLC_Vector4d(0.1, 0.0, 0.0));
		m_GlView.m_pViewCam->setDistEyeTarget(distance);
	}
	GLC_Matrix4x4 matRot(m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget(), 5 * glc::EPSILON);
	GLC_Vector4d newUp(matRot * m_GlView.m_pViewCam->getVectUp());
	m_GlView.m_pViewCam->setUpCam(newUp);
	
	eye= eye + target;
	// Up vector
	const GLC_Vector4d vectUp(0.0, 1.0, 0.0);
	GLC_Camera newCam(eye, target, vectUp);
	newCam.setDistEyeTarget(m_GlView.m_pViewCam->getDistEyeTarget());
	rotateCamera(newCam, m_MotionStep);
	
}
// Left view
void OpenglView::leftView()
{
	const GLC_Point4d target(m_GlView.m_pViewCam->getTarget());
	const double distance= m_GlView.m_pViewCam->getDistEyeTarget();
	// Define position of the camera eye
	GLC_Point4d eye(-1.0, 0.0, 0.0);
	// Check angle
	const GLC_Vector4d vectCam((m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget()));
	const double Angle= acos(vectCam * eye);
	if ((glc::PI - Angle) < glc::EPSILON)
	{
		m_GlView.m_pViewCam->setEyeCam(m_GlView.m_pViewCam->getEye() + GLC_Vector4d(0.0, 0.0, 0.1));
		m_GlView.m_pViewCam->setDistEyeTarget(distance);
	}
	GLC_Matrix4x4 matRot(m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget(), 5 * glc::EPSILON);
	GLC_Vector4d newUp(matRot * m_GlView.m_pViewCam->getVectUp());
	m_GlView.m_pViewCam->setUpCam(newUp);
	
	eye= eye + target;
	// Up vector
	const GLC_Vector4d vectUp(0.0, 1.0, 0.0);
	GLC_Camera newCam(eye, target, vectUp);
	newCam.setDistEyeTarget(m_GlView.m_pViewCam->getDistEyeTarget());
	rotateCamera(newCam, m_MotionStep);
	
}
// Right view
void OpenglView::rightView()
{
	const GLC_Point4d target(m_GlView.m_pViewCam->getTarget());
	const double distance= m_GlView.m_pViewCam->getDistEyeTarget();
	// Define position of the camera eye
	GLC_Point4d eye(1.0, 0.0, 0.0);
	// Check angle
	const GLC_Vector4d VectCam((m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget()).setNormal(1));
	const double Angle= acos(VectCam * eye);
	if ((glc::PI - Angle) < glc::EPSILON)
	{
		m_GlView.m_pViewCam->setEyeCam(m_GlView.m_pViewCam->getEye() + GLC_Vector4d(0.0, 0.0, 0.1));
		m_GlView.m_pViewCam->setDistEyeTarget(distance);
		
	}
	GLC_Matrix4x4 matRot(m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget(), 5 * glc::EPSILON);
	GLC_Vector4d newUp(matRot * m_GlView.m_pViewCam->getVectUp());
	m_GlView.m_pViewCam->setUpCam(newUp);
	
	eye= eye + target;
	// Up vector
	const GLC_Vector4d vectUp(0.0, 1.0, 0.0);
	GLC_Camera newCam(eye, target, vectUp);
	newCam.setDistEyeTarget(m_GlView.m_pViewCam->getDistEyeTarget());
	rotateCamera(newCam, m_MotionStep);
	
}
// Top view
void OpenglView::topView()
{
	const GLC_Point4d target(m_GlView.m_pViewCam->getTarget());
	const double distance= m_GlView.m_pViewCam->getDistEyeTarget();
	// Define position of the camera eye
	GLC_Point4d eye(0.0, 1.0, 0.0);
	// Check angle
	const GLC_Vector4d VectCam((m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget()).setNormal(1));
	const double Angle= acos(VectCam * eye);
	if ((glc::PI - Angle) < glc::EPSILON)
	{
		m_GlView.m_pViewCam->setEyeCam(m_GlView.m_pViewCam->getEye() + GLC_Vector4d(0.0, 0.0, 0.1));
		m_GlView.m_pViewCam->setDistEyeTarget(distance);
	}
	GLC_Matrix4x4 matRot(m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget(), 5 * glc::EPSILON);
	GLC_Vector4d newUp(matRot * m_GlView.m_pViewCam->getVectUp());
	m_GlView.m_pViewCam->setUpCam(newUp);
	
	eye= eye + target;
	// Up vector
	const GLC_Vector4d vectUp(0.0, 0.0, -1.0);	
	GLC_Camera newCam(eye, target, vectUp);
	newCam.setDistEyeTarget(m_GlView.m_pViewCam->getDistEyeTarget());
	rotateCamera(newCam, m_MotionStep);
	
}
// Bottom view
void OpenglView::bottomView()
{
	const GLC_Point4d target(m_GlView.m_pViewCam->getTarget());
	const double distance= m_GlView.m_pViewCam->getDistEyeTarget();
	// Define position of the camera eye
	GLC_Point4d eye(0.0, -1.0, 0.0);
	// Check angle
	const GLC_Vector4d VectCam((m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget()).setNormal(1));
	const double Angle= acos(VectCam * eye);
	if ((glc::PI - Angle) < glc::EPSILON)
	{
		m_GlView.m_pViewCam->setEyeCam(m_GlView.m_pViewCam->getEye() + GLC_Vector4d(0.0, 0.0, 0.1));
		m_GlView.m_pViewCam->setDistEyeTarget(distance);
	}
	GLC_Matrix4x4 matRot(m_GlView.m_pViewCam->getEye() - m_GlView.m_pViewCam->getTarget(), 5 * glc::EPSILON);
	GLC_Vector4d newUp(matRot * m_GlView.m_pViewCam->getVectUp());
	m_GlView.m_pViewCam->setUpCam(newUp);
	
	eye= eye + target;
	// Up vector
	const GLC_Vector4d vectUp(0.0, 0.0, 1.0);
	GLC_Camera newCam(eye, target, vectUp);
	newCam.setDistEyeTarget(m_GlView.m_pViewCam->getDistEyeTarget());
	rotateCamera(newCam, m_MotionStep);
	
}
// Zoom in
void OpenglView::zoomIn()
{
	int max= static_cast<int>(static_cast<double>(m_MotionStep) / 4);
	if (max == 0) max= 1;
	const double targetFactor= 1.5;
	const double factor= exp(log(targetFactor)/ max);
	
	for (int i= 0; i < max; ++i)
	{
		m_GlView.m_pViewCam->zoom(factor);
		const GLC_BoundingBox collectionBox= m_World.collection()->getBoundingBox();
		m_GlView.setDistMinAndMax(collectionBox);
		updateGL();
		emit viewChanged();
	}
}
// Zoom out
void OpenglView::zoomOut()
{
	int max= static_cast<int>(static_cast<double>(m_MotionStep) / 4);
	if (max == 0) max= 1;
	const double targetFactor= 1.0 / 1.5;
	const double factor= exp(log(targetFactor)/ max);
	
	for (int i= 0; i < max; ++i)
	{
		m_GlView.m_pViewCam->zoom(factor);
		const GLC_BoundingBox collectionBox= m_World.collection()->getBoundingBox();
		m_GlView.setDistMinAndMax(collectionBox);
		updateGL();
		emit viewChanged();
	}
}

// unselect Instance
void OpenglView::unselect()
{
	// if a geometry is selected, unselect it
	m_World.collection()->unselectAll();
	updateGL();
	emit unselectGeom();
}

//////////////////////////////////////////////////////////////////////
// Private services Functions
//////////////////////////////////////////////////////////////////////
void OpenglView::paintGL()
{
	QTime time;
	time.start();
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	// Enable and execute lighting
	m_Light.enable();
	try
	{
		m_Light.glExecute();
		m_GlView.glExecuteCam();
		m_World.collection()->glExecute();
	}
	catch (GLC_Exception &e)
	{
		qDebug() << e.what();
	}	

	if (!m_SnapShootMode) // Don't display orbit circle in snapshootmode
	{
		m_GlView.glExecute();
	}
	updateFps(time.elapsed());
	if (!m_SelectionMode && m_dislayInfoPanel && !m_SnapShootMode)
	{
		// Display info area	
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho(-1,1,-1,1,-1,1);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();
		glPushAttrib(GL_ENABLE_BIT);
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		
		displayInfo();	
	
		// Restore 3DState
		glPopAttrib();
		glPopMatrix(); // restore modelview
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
	}
}


void OpenglView::resizeGL(int width, int height)
{
	m_GlView.setWinGLSize(width, height);
}

void OpenglView::mousePressEvent(QMouseEvent * e)
{
	if ((m_ViewState == V_NORMAL) && (m_ViewEnterState == VE_NORMAL))
	{
		switch (e->button())
		{
			// Left Button Pressed
			case Qt::LeftButton:
				select(e->x(),e->y());
			break;
			
			// Right Button Pressed
			case Qt::RightButton:
			if (e->modifiers() == Qt::NoModifier)
			{
				m_GlView.prepareOrbiting(e->x(),e->y());
				setCursor(Qt::ClosedHandCursor);
				m_ViewState= V_ORBITING;
				updateGL();
			}
			else if (e->modifiers() == Qt::ShiftModifier)
			{
				m_GlView.prepareZooming(e->y());
				setCursor(Qt::SizeVerCursor);
				m_ViewState= V_ZOOMING;
				updateGL();
			}
			break;
			
			// Mid Button Pressed
			case Qt::MidButton:
			if (e->modifiers() == Qt::NoModifier)
			{
				m_GlView.preparePanning(e->x(), e->y());
				setCursor(Qt::SizeAllCursor);
				m_ViewState= V_PANNING;
				updateGL();
			}
			else if (e->modifiers() == Qt::ShiftModifier)
			{
				m_GlView.glPointing(e->x(), e->y());
				updateGL();
				emit viewChanged();
			}
			
			break;
			// Default case
			default:
			break;
		}
	}
	else if ((m_ViewState == V_NORMAL) && (e->button() == Qt::LeftButton))
	{
		switch (m_ViewEnterState)
		{
			case VE_PANNING:
			m_GlView.preparePanning(e->x(), e->y());
			setCursor(Qt::SizeAllCursor);
			m_ViewState= V_PANNING;
			updateGL();			
			break;
			
			case VE_ORBITING:
			m_GlView.prepareOrbiting(e->x(),e->y());
			setCursor(Qt::ClosedHandCursor);
			m_ViewState= V_ORBITING;
			updateGL();			
			break;
			
			case VE_POINTING:
			m_GlView.glPointing(e->x(), e->y());
			updateGL();	
			break;
			
			case VE_ZOOMING:
			m_GlView.prepareZooming(e->y());
			setCursor(Qt::SizeVerCursor);
			m_ViewState= V_ZOOMING;
			updateGL();			
			break;
			
			default:
			break;
			
		}
	}
	
	QGLWidget::mousePressEvent(e);
}

void OpenglView::mouseReleaseEvent(QMouseEvent * e)
{
	if (m_ViewEnterState == VE_NORMAL)
	{
		switch (e->button())
		{
			// Left Button Released
			case Qt::LeftButton:
			// Perhaps selection...
			break;
			
			// Right Button Released
			case Qt::RightButton:
			if (m_ViewState == V_ORBITING)
			{
				m_GlView.endOrbit();
				unsetCursor();
				m_ViewState= V_NORMAL;
				updateGL();
			}
			else if (m_ViewState == V_ZOOMING)
			{
				m_GlView.EndZooming();
				unsetCursor();
				m_ViewState= V_NORMAL;
				updateGL();
			}
			break;
			
			// Mid Button Released
			case Qt::MidButton:
			if (m_ViewState == V_PANNING)
			{
				m_GlView.endPanning();
				unsetCursor();
				m_ViewState= V_NORMAL;
				updateGL();
			}
			break;
			
			// Default case
			default:
			break;
		}
	}
	else if(e->button() == Qt::LeftButton)
	{
		switch (m_ViewEnterState)
		{
			case VE_PANNING:
			m_GlView.endPanning();
			unsetCursor();
			m_ViewState= V_NORMAL;
			updateGL();
			break;
			
			case VE_ORBITING:
			m_GlView.endOrbit();
			unsetCursor();
			m_ViewState= V_NORMAL;
			updateGL();
			break;

			case VE_ZOOMING:
			m_GlView.EndZooming();
			unsetCursor();
			m_ViewState= V_NORMAL;
			updateGL();
			break;
			
			default:
			break;
		}
	}
	
}
void OpenglView::mouseMoveEvent(QMouseEvent * e)
{
	switch (m_ViewState)
	{
		case V_ORBITING:
		m_GlView.orbit(e->x(),e->y());
		// Calculate camera depth of view
		m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
		updateGL();
		emit viewChanged();
		break;
		
		case V_ZOOMING:
		m_GlView.zoom(e->y());
		// Calculate camera depth of view
		m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());		
		updateGL();
		emit viewChanged();
		break;
		
		case V_PANNING:
		m_GlView.pan(e->x(),e->y());
		updateGL();
		emit viewChanged();
		break;
		
		default:
		break;
	}
}
void OpenglView::wheelEvent(QWheelEvent *e)
{
	if (e->delta() > 0)
	{
		zoomIn();
	}
	else
	{
		zoomOut();
	}
}
void OpenglView::mouseDoubleClickEvent(QMouseEvent *e)
{
	if ((m_World.collection()->getNumberOfSelectedNode() > 0) and (e->button() == Qt::LeftButton))
	{
		GLC_BoundingBox SelectionBox= m_World.collection()->getSelections()->begin().value()->getBoundingBox();
		reframe(SelectionBox, true);
	}
}

// Rotate the camera softely with specified step
void OpenglView::rotateCamera(const GLC_Camera newCam, const int step)
{
	if ((m_GlView.m_pViewCam->getEye() != newCam.getEye()) || 
		(m_GlView.m_pViewCam->getVectUp() != newCam.getVectUp()))
		{
			GLC_Point4d curEye(m_GlView.m_pViewCam->getEye());
			GLC_Vector4d vectCurUp(m_GlView.m_pViewCam->getVectUp());
			GLC_Point4d curTarget(m_GlView.m_pViewCam->getTarget());
			
			GLC_Interpolator angleInterpolator;
			angleInterpolator.SetInterpolMat(step, (curEye - curTarget) , (newCam.getEye() - newCam.getTarget()), INTERPOL_ANGULAIRE);
			GLC_Matrix4x4 matRot(angleInterpolator.GetInterpolMat());
			
			// VecteurUp
			angleInterpolator.SetVecteurs(vectCurUp, newCam.getVectUp());
			GLC_Matrix4x4 matRotUp(angleInterpolator.GetInterpolMat());
	
			for (int i= 0; i < step - 1; ++i)
			{
				curEye= (matRot * (curEye - curTarget)) + curTarget;
				vectCurUp= matRotUp * vectCurUp;
	
				m_GlView.m_pViewCam->setCam(curEye, curTarget, vectCurUp);
				m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
				updateGL();
			}		
			m_GlView.m_pViewCam->setCam(newCam.getEye(), newCam.getTarget(), newCam.getVectUp());
			m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
			updateGL();
			emit viewChanged();
		}
		
}
// reframe softely with specified step
void OpenglView::motionReframe(const GLC_Camera newCam, const int step)
{
	if ((m_GlView.m_pViewCam->getEye() != newCam.getEye()) || 
		(m_GlView.m_pViewCam->getTarget() != newCam.getTarget()))
		{
			GLC_Point4d curEye(m_GlView.m_pViewCam->getEye());
			GLC_Point4d curTarget(m_GlView.m_pViewCam->getTarget());
			const GLC_Vector4d vectCurUp(m_GlView.m_pViewCam->getVectUp());
			
			GLC_Interpolator transInterpolator;
			transInterpolator.SetInterpolMat(step, curEye, newCam.getEye(), INTERPOL_LINEAIRE);
			GLC_Matrix4x4 matEye(transInterpolator.GetInterpolMat());
			transInterpolator.SetVecteurs(curTarget, newCam.getTarget());
			GLC_Matrix4x4 matTarget(transInterpolator.GetInterpolMat());
			for (int i= 0; i < step - 1; ++i)
			{
				curEye= matEye * curEye;
				curTarget = matTarget * curTarget;
	
				m_GlView.m_pViewCam->setCam(curEye, curTarget, vectCurUp);
				m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
				updateGL();
			}
			m_GlView.m_pViewCam->setCam(newCam.getEye(), newCam.getTarget(), newCam.getVectUp());
			m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
			updateGL();
			emit viewChanged();
						
		}
}
// Update the current fps
void OpenglView::updateFps(int elapsed)
{
	static const int max= 20;
	static int fpsVector[max];
	static int j=0;
	float averageFps= 0.0f;
	
	
  	if (elapsed > 0)
  	{
    	fpsVector[j]= elapsed;
	  	j= (j + 1) % max;
  	}
	for (int i= 0;i < max;++i)
	{
		averageFps+= fpsVector[i];
	}
	averageFps= averageFps / static_cast<float>(max);
	m_CurrentFps= static_cast<int>(1000.0f / averageFps);
	int relativeFps;
	
	// The maximum relative fps must be under 60
	// Cause VSSYNC
	if (m_CurrentFps > 60) relativeFps= 60;
	else relativeFps= m_CurrentFps;

	m_MotionStep= static_cast<int>(static_cast<float>(relativeFps) / 3);
	if (0 == m_MotionStep) m_MotionStep= 1;
}

// Display info panel
void OpenglView::displayInfo()
{
	QSize textSize(70,5);
	QSize screenSize(size());
	int screenHeight= screenSize.height();
	float panelRatio= static_cast<float>(screenHeight - 42) / screenHeight;
	
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glBlendFunc(GL_ONE,GL_SRC_ALPHA);
	QColor areaColor(5, 5, 50, 128);
	qglColor(areaColor);
	
	glBegin(GL_QUADS);
		glVertex2f(-1.f,-panelRatio); glVertex2f( 1.f,-panelRatio);
		glVertex2f( 1.f,-1.f); glVertex2f(-1.f,-1.f);
	glEnd();	
	
	// Display the fps
	
	qglColor(Qt::white);
	m_infoFont.setPixelSize(12);
	screenSize-= textSize;
	QString fps(QString::number(m_CurrentFps) + QString(" Fps"));
	renderText(screenSize.width(), screenSize.height(), fps, m_infoFont);
	
	// OpenGL error handler
	GLenum error= glGetError();	
	if (error != GL_NO_ERROR)
	{// QT render text cause an openGL error
		QString msg(tr("Sorry, Fps info panel not supported on your computer, restarting GLC_Player"));
		QMessageBox::warning(this, tr("Warning"), msg, QMessageBox::Ok);
		m_dislayInfoPanel= false;
		emit hideInfoPanel();
	}
	
}

// Select
void OpenglView::select(int x, int y)
{
	m_SelectionMode= true;
	setAutoBufferSwap(false);
	GLC_uint SelectionID= m_GlView.select(this, x, y);
	m_SelectionMode= false;
	updateGL();
	setAutoBufferSwap(true);

	if (m_World.collection()->isInCollection(SelectionID))
	{
		
		if ((!m_World.collection()->isSelected(SelectionID)) && (m_World.collection()->getNumberOfSelectedNode() > 0))
		{
			m_World.collection()->unselectAll();
			emit unselectGeom();
		}
		if (!m_World.collection()->isSelected(SelectionID))
		{
			m_World.collection()->select(SelectionID);
			updateGL();
			GLC_Instance* pInstance= m_World.collection()->getInstanceHandle(SelectionID);
			emit selectInstance(pInstance);
			
		}
	}
	else if (m_World.collection()->getNumberOfSelectedNode() > 0)
	{
		// if a geometry is selected, unselect it
		m_World.collection()->unselectAll();
		updateGL();
		emit unselectGeom();
	}

}
void OpenglView::setDistMinAndMax()
{
	m_GlView.setDistMinAndMax(m_World.collection()->getBoundingBox());
}
// Swap visible space
void OpenglView::swapVisibleSpace()
{
	m_World.collection()->swapShowState();
	if (m_World.collection()->getShowState())
	{
		m_GlView.loadBackGroundImage(":images/default_background.png");
	}
	else
	{
		m_GlView.loadBackGroundImage(":images/NoShow_background.png");
	}
}
// set the view to visible state
void OpenglView::setToVisibleState()
{
	m_GlView.loadBackGroundImage(":images/default_background.png");
}
// set the view to visible state
void OpenglView::setToInVisibleState()
{
	m_GlView.loadBackGroundImage(":images/NoShow_background.png");
}




