/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 1.6.0, packaged on August 2008.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "glc_player.h"
#include "SettingsDialog.h"
#include "AboutPlayer.h"
#include "EditMeshDialog.h"
#include "AlbumFile.h"
#include "EditCamera.h"
#include "SelectionProperty.h"
#include "EditLightDialog.h"

#include <GLC_Exception>

glc_player::glc_player(QWidget *parent)
: QMainWindow(parent)
, m_OpenglView(this)
, m_CurrentPath(QDir::homePath())
, m_CurrentAlbumPath(QDir::homePath())
, m_pProgressBar(new QProgressBar())
, m_pQErrorMessage(NULL)
, m_RecentFilesList()
, m_RecentAlbumsList()
, m_CurrentFileName()
, m_CurrentAlbumName()
, m_OpenFileThread(GLC_Factory::instance(m_OpenglView.context()))
, m_FileLoadingInProgress(false)
, m_ListLoadingInProgress(false)
, m_ContinuListLoading(true)
, m_NumberOfUnloadedFile(0)
, m_NumberOfErrorFile(0)
, m_iconSize()
, m_MakeFirstFileCurrent(true)
, m_DisplayThumbnails(true)
, m_pActionDeleteSelectedItem(new QAction(tr("Remove Model"), this))
, m_pActionGetErrorInfo(new QAction(tr("Get Error Information"), this))
, m_pCurrentSelection(NULL)
, m_dislayInfoPanel()
, m_pSelectionProperty(NULL)
, m_pEditLightDialog(NULL)
, m_QuitConfirmation()
{
	ui.setupUi(this);
	setCentralWidget(&m_OpenglView);
	m_OpenglView.setFocusPolicy(Qt::StrongFocus);
	readSettings();
	createRecentFileActionsArray();
	updateRecentsFiles();
	createRecentAlbumActionsArray();
	updateRecentsAlbums();
	
	// Default lighting
	m_OpenglView.getLight()->setTwoSided(ui.actionTwo_sided_Lightning->isChecked());
	
	// Display info panel
	m_OpenglView.setDisplayInfoPanel(m_dislayInfoPanel);
	
	// Resize Texture cache limite to 128 Mo
	QGLContext::setTextureCacheLimit(128 * 1024);
	
	// Item list Icons
	ui.loadingList->setIconSize(m_iconSize);

	// Accept drop event
	m_OpenglView.setAcceptDrops(false);
	setAcceptDrops(true);
	
	// Set the current file name
	addToRecentFiles(m_CurrentFileName);	
	
	// Status bar
	ui.statusbar->addPermanentWidget(m_pProgressBar);
	m_pProgressBar->hide();	
	connect(&m_OpenglView, SIGNAL(currentQuantum(int)), this, SLOT(updateProgressBar(int)), Qt::QueuedConnection);
		
	// QAction Signals and slots connection
	// Signals from the view
	connect(&m_OpenglView, SIGNAL(selectInstance(GLC_Instance*)), this, SLOT(select(GLC_Instance*)));
	connect(&m_OpenglView, SIGNAL(unselectGeom()), this, SLOT(unselect()));
	connect(&m_OpenglView, SIGNAL(hideInfoPanel()), this, SLOT(hideInfoPanel()));
	//Menu File
	connect(ui.action_NewAlbum, SIGNAL(triggered()), this , SLOT(newAlbum()));
	connect(ui.action_OpenAlbum, SIGNAL(triggered()), this , SLOT(openAlbum()));
	connect(ui.action_SaveAlbum, SIGNAL(triggered()), this , SLOT(saveAlbum()));
	connect(ui.action_SaveAlbumAs, SIGNAL(triggered()), this , SLOT(saveAlbumAs()));
	connect(ui.action_Open, SIGNAL(triggered()), this, SLOT(open()));
	connect(ui.action_Quit, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));
	// Menu edit
	connect(ui.actionUnselect, SIGNAL(triggered()), &m_OpenglView, SLOT(unselect()));
	connect(ui.action_Property, SIGNAL(triggered()), this, SLOT(meshProperty()));
	//Menu Window
	connect(ui.action_AlbumManagement, SIGNAL(triggered()), this, SLOT(albumManagementVisibilityToggle()));
	connect(ui.albumManagementWindow, SIGNAL(visibilityChanged(bool)), this, SLOT(albumManagementVisibilityChanged(bool)));
	connect(ui.action_CameraProperty, SIGNAL(triggered()), this, SLOT(cameraPropertyVisibilityToggle()));
	connect(ui.cameraProperty, SIGNAL(visibilityChanged(bool)), this, SLOT(cameraPropertyVisibilityChanged(bool)));
	connect(ui.action_SelectionProperty, SIGNAL(triggered()), this, SLOT(selectionPropertyVisibilityToggle()));
	connect(ui.selectionDockWidget, SIGNAL(visibilityChanged(bool)), this, SLOT(selectionPropertyVisibilityChanged(bool)));

	//Menu View
	connect(ui.action_Reframe, SIGNAL(triggered()), this, SLOT(reframe()));
	connect(ui.action_Select, SIGNAL(triggered()), this, SLOT(selectMode()));
	connect(ui.action_ViewCenter, SIGNAL(triggered()), this, SLOT(viewCenterMode()));
	connect(ui.action_Pan, SIGNAL(triggered()), this, SLOT(panMode()));
	connect(ui.action_Rotate, SIGNAL(triggered()), this, SLOT(rotateMode()));
	connect(ui.action_Zoom, SIGNAL(triggered()), this, SLOT(zoomMode()));
	connect(ui.action_ZoomIn, SIGNAL(triggered()), &m_OpenglView, SLOT(zoomIn()));
	connect(ui.action_ZoomOut, SIGNAL(triggered()), &m_OpenglView, SLOT(zoomOut()));
	connect(ui.actionShow_Hide, SIGNAL(triggered()), this, SLOT(showOrHide()));
	connect(ui.actionHide_unselected, SIGNAL(triggered()), this, SLOT(hideUnselected()));
	connect(ui.actionShow_all, SIGNAL(triggered()), this, SLOT(showAll()));
	connect(ui.actionSwap_visible_space, SIGNAL(triggered()), this, SLOT(swapVisibleSpace()));
	connect(ui.action_FullScreen, SIGNAL(triggered()), this, SLOT(fullScreen()));
	
	// predifined view 
	connect(ui.action_IsoView, SIGNAL(triggered()), &m_OpenglView, SLOT(isoView()));
	connect(ui.action_FrontView, SIGNAL(triggered()), &m_OpenglView, SLOT(frontView()));
	connect(ui.action_BackView, SIGNAL(triggered()), &m_OpenglView, SLOT(backView()));
	connect(ui.action_LeftView, SIGNAL(triggered()), &m_OpenglView, SLOT(leftView()));
	connect(ui.action_RightView, SIGNAL(triggered()), &m_OpenglView, SLOT(rightView()));
	connect(ui.action_TopView, SIGNAL(triggered()), &m_OpenglView, SLOT(topView()));
	connect(ui.action_BottomView, SIGNAL(triggered()), &m_OpenglView, SLOT(bottomView()));
	
	// Menu Render
	// Render Mode
	connect(ui.action_RenderPoints, SIGNAL(triggered()), this, SLOT(pointsRenderingMode()));
	connect(ui.action_RenderWireframe, SIGNAL(triggered()), this, SLOT(wireframeRenderingMode()));
	connect(ui.action_RenderShading, SIGNAL(triggered()), this, SLOT(shadingRenderingMode()));
	connect(ui.action_EditLight, SIGNAL(triggered()), this, SLOT(editLightDialog()));
	connect(ui.actionTwo_sided_Lightning, SIGNAL(triggered()), this, SLOT(twoSidedLightning()));
	// Menu Tools
	connect(ui.action_Settings, SIGNAL(triggered()), this, SLOT(showSettings()));
	connect(ui.action_SnapShot, SIGNAL(triggered()), this, SLOT(takeSnapShot()));
	
	// Menu Help
	connect(ui.action_About, SIGNAL(triggered()), this, SLOT(aboutPlayer()));
	connect(ui.action_Help, SIGNAL(triggered()), this, SLOT(help()));
	
	// Open File thread
	connect(&m_OpenFileThread, SIGNAL(currentQuantum(int)), this, SLOT(updateProgressBar(int)), Qt::QueuedConnection);
	connect(&m_OpenFileThread, SIGNAL(finished()), this, SLOT(fileOpened()), Qt::QueuedConnection);
	connect(&m_OpenFileThread, SIGNAL(loadError()), this, SLOT(loadFileFailed()), Qt::QueuedConnection);
	
	// Item list dock window
	connect(ui.displayItemCheckBox, SIGNAL(stateChanged(int)), this , SLOT(manageItemInformationDisplay(int)));
	connect(ui.refreshThumbnails, SIGNAL(clicked()), this , SLOT(refreshIconInItemList()));
	connect(ui.removeUnloadItemButton, SIGNAL(clicked()), this , SLOT(removeUnloadFileItem()));
	connect(ui.removeOnErrorItem, SIGNAL(clicked()), this , SLOT(removeItemOnError()));
	connect(ui.startLoadingButton, SIGNAL(clicked()), this , SLOT(startLoadingButton()));
	connect(ui.stopLoadingButton, SIGNAL(clicked()), this , SLOT(stopLoadingButton()));
	connect(ui.reverseNormals, SIGNAL(clicked()), this, SLOT(reverseModelNormal()));
	
	// List Widget
	connect(ui.loadingList, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *))
			, this , SLOT(currentFileItemChanged(QListWidgetItem *, QListWidgetItem *)));
	connect(ui.loadingList, SIGNAL(itemClicked(QListWidgetItem *)), this , SLOT(fileItemClicked(QListWidgetItem *)));
	
	
	//Contextual menu Creation
	ui.loadingList->addAction(m_pActionDeleteSelectedItem);
	QList<QKeySequence> keys;
	keys.append(Qt::Key_Delete);
	keys.append(Qt::Key_Backspace);
	m_pActionDeleteSelectedItem->setShortcuts(keys);
	connect(m_pActionDeleteSelectedItem, SIGNAL(triggered()), this, SLOT(deleteCurrentItem()));
	m_pActionDeleteSelectedItem->setEnabled(false);
	// Connect action get error information
	connect(m_pActionGetErrorInfo, SIGNAL(triggered()), this, SLOT(getErrorInformation()));
	
	// Camera Property dock area
	EditCamera* pEditCamWidget= new EditCamera(m_OpenglView.getViewport(), ui.cameraProperty);	
	ui.cameraProperty->setWidget(pEditCamWidget);
	connect(&m_OpenglView, SIGNAL(viewChanged()), pEditCamWidget, SLOT(updateValues()));
	connect(pEditCamWidget, SIGNAL(valueChanged()), this, SLOT(updateView()));
	
	// Current selection dock area
	m_pSelectionProperty= new SelectionProperty(ui.selectionDockWidget);
	ui.selectionDockWidget->setWidget(m_pSelectionProperty);
	connect(m_pSelectionProperty, SIGNAL(updateView()), &m_OpenglView, SLOT(updateGL()));
	connect(m_pSelectionProperty, SIGNAL(showOrHide()), this, SLOT(showOrHide()));
	// Parse arguments command line
	QStringList args= QCoreApplication::arguments ();
	if (args.size() > 1)
	{
		QStringList argList(QFileInfo(args[1]).filePath());
		if (QFileInfo(argList[0]).suffix().toLower() == "album")
		{
			openAlbum(argList[0]);
		}
		else
		{
			addItems(argList);
			startLoading();
		}
	}
	else
	{
		updateUIState();
	}
}

glc_player::~glc_player()
{
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	// delete the progress bar
	if (NULL != m_pProgressBar)
	{
		delete m_pProgressBar;
	}
	
	// delete error message dialog
	if (NULL != m_pQErrorMessage)
	{
		delete m_pQErrorMessage;
	}
	
	if (m_pEditLightDialog != NULL) delete m_pEditLightDialog;
	
	ui.loadingList->clear();
	m_FileEntryHash.clear();
	QApplication::restoreOverrideCursor();
}
//////////////////////////////////////////////////////////////////////
// Virtual protected function
//////////////////////////////////////////////////////////////////////
// Close event handler
void glc_player::closeEvent(QCloseEvent* pEvent)
{
	int ret= QMessageBox::Yes;
	// Display confirmation msgBox only if there album is not empty
	// And Quit confirmation is set to yes
	if ((ui.loadingList->count() > 0) and m_QuitConfirmation)
	{
		ret= QMessageBox::question(this, QCoreApplication::applicationName(),
				QString(tr("Quits ")) +  QCoreApplication::applicationName() + QString("?"), QMessageBox::Yes | QMessageBox::No);
	}
	if (ret == QMessageBox::Yes)
	{
		ui.loadingList->blockSignals(true);
		writeSettings();
		pEvent->accept();
		QCoreApplication::quit();
	}
	else
	{
		pEvent->ignore();
	}
}

//////////////////////////////////////////////////////////////////////
// public Methods
//////////////////////////////////////////////////////////////////////
void glc_player::openOnEvent(QString fileName)
{
	
	if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
	const bool fileIsAlbum= (QFileInfo(fileName).suffix().toLower() == "album");
	if (fileIsAlbum and ui.action_OpenAlbum->isEnabled())
	{
		openAlbum(fileName);
	}
	else if (not fileIsAlbum)
	{
		QStringList localFiles;
		localFiles.append(fileName);
		addItems(localFiles);
		startLoading();
	}

}

//////////////////////////////////////////////////////////////////////
// public Slots
//////////////////////////////////////////////////////////////////////
// an instance as been selected
void glc_player::select(GLC_Instance* pInstance)
{
	m_pCurrentSelection= pInstance;
	m_pSelectionProperty->setSelection(m_pCurrentSelection);
	ui.action_Property->setEnabled(true);
	ui.actionUnselect->setEnabled(true);
	ui.actionShow_Hide->setEnabled(true);
	ui.actionHide_unselected->setEnabled(true);
}

// an instance as been unselected
void glc_player::unselect()
{
	m_pCurrentSelection= NULL;
	m_pSelectionProperty->unsetSelection();
	ui.action_Property->setEnabled(false);
	ui.actionUnselect->setEnabled(false);
	ui.actionShow_Hide->setEnabled(false);
	ui.actionHide_unselected->setEnabled(false);
}

// Info panel not supported
void glc_player::hideInfoPanel()
{
	m_dislayInfoPanel= false;
	writeSettings();
	QCoreApplication::quit();	
}
//////////////////////////////////////////////////////////////////////
// private Slots
//////////////////////////////////////////////////////////////////////

// Set Instance visibility
void glc_player::showOrHide()
{
	if ((NULL != ui.loadingList->currentItem()) && (NULL!= m_pCurrentSelection))
	{
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		// Take the entry from hash table
		m_FileEntryHash.value(fileName).getWorld().collection()->setVisibility(m_pCurrentSelection->getID(), !m_pCurrentSelection->isVisible());
		m_OpenglView.updateGL();
	}
	
}
// Hide unseleted instance
void glc_player::hideUnselected()
{
	if ((NULL != ui.loadingList->currentItem()) && (NULL!= m_pCurrentSelection))
	{
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		// Take the entry from hash table
		m_FileEntryHash.value(fileName).getWorld().collection()->hideAll();
		m_FileEntryHash.value(fileName).getWorld().collection()->setVisibility(m_pCurrentSelection->getID(), true);
		m_OpenglView.updateGL();
	}
	
}

// Show All instance
void glc_player::showAll()
{
	if (NULL != ui.loadingList->currentItem())
	{
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		// Show all model of the entry collection
		m_FileEntryHash.value(fileName).getWorld().collection()->showAll();
		m_OpenglView.updateGL();
	}
	
}

// Swap visible space
void glc_player::swapVisibleSpace()
{
	if (NULL != ui.loadingList->currentItem())
	{
		m_OpenglView.swapVisibleSpace();
		m_OpenglView.updateGL();
	}	
}

// update the value of the progress bar
void glc_player::updateProgressBar(int value)
{
	if (value < 100)
	{
		ui.statusbar->showMessage(tr("Loading in Progress Please Wait"));
		m_pProgressBar->setValue(value);
	}
	else
	{
		ui.statusbar->showMessage(tr("File Loaded"));
		m_pProgressBar->hide();
	}
}


//////////////////////////////////////////////////////////////////////
// private Slots
//////////////////////////////////////////////////////////////////////

// Remove all file item from the current album
bool glc_player::newAlbum(bool confirmation)
{
	if (!m_FileLoadingInProgress)
	{
		int ret= QMessageBox::No;
		if (confirmation)
		{
			ret= QMessageBox::question(this, tr("New Album Confirmation"),
								 	tr("Remove all models from the current album?"), QMessageBox::Yes | QMessageBox::No);
		}
		if (!confirmation || (ret == QMessageBox::Yes))
		{
			QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
			unselect();
			//ui.loadingList->blockSignals(true);	
			ui.loadingList->clear();
			//ui.loadingList->blockSignals(false);
			m_FileEntryHash.clear();
			
			// Update number of loaded file and error
			m_NumberOfErrorFile= 0;
			m_NumberOfUnloadedFile= 0;
			// Update Continu list loading flag
			m_ContinuListLoading= true;
			// Update UI
			m_CurrentAlbumName.clear();
			updateUIState();
			QApplication::restoreOverrideCursor();
			ui.loadingList->removeAction(m_pActionGetErrorInfo);
			return true;
		}
		else return false;
	}
	else return false;
}

// Open An existing file
void glc_player::open()
{
	// Define File Format filter
	QStringList filters;
	filters.append(tr("All Known format(*.obj *.OBJ *.3ds *.3DS *.stl *.STL *.off *.OFF)"));
	filters.append(tr("Alias File Format OBJ (*.obj *.OBJ)"));
	filters.append(tr("3D Studio File Format 3DS (*.3ds *.3DS)"));
	filters.append(tr("STL File Format STL (*.stl *.STL)"));
	filters.append(tr("Object File Format OFF (*.off *.OFF)"));
	
	QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Select File(s) to Open and Add in album")
	, m_CurrentPath, filters.join("\n"));
	if (!fileNames.isEmpty())
	{
		if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
		addItems(fileNames);
		startLoading();
	}
}
// Open An existing album
void glc_player::openAlbum()
{
	if (!m_FileLoadingInProgress)
	{
		QString fileName = QFileDialog::getOpenFileName(this, tr("Select Album File to Open")
		, m_CurrentAlbumPath, "GLC_Player Album (*.album)");
		if (!fileName.isEmpty())
		{
			openAlbum(fileName);
		}
	}
}
// Save the current album
void glc_player::saveAlbum()
{
	if (m_CurrentAlbumName.isEmpty())
	{
		saveAlbumAs();
	}
	else
	{
		applySavingAlbum(m_CurrentAlbumName);
	}
	
}

// Save As a new album
void glc_player::saveAlbumAs()
{
	QString currentPath;
	if (!m_CurrentAlbumName.isEmpty())
	{
		currentPath= m_CurrentAlbumName;
	}
	else
	{
		currentPath= m_CurrentAlbumPath;
	}
	const QString suffix(".album");
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save Album As ")
	, currentPath, tr("GLC_Player Album (*.album)"));
	if (!fileName.isEmpty())
	{
		if (!fileName.endsWith(suffix))
		{
			fileName.append(suffix);
		}
		applySavingAlbum(fileName);

	}
}

// Open a Recent File
void glc_player::openRecentFile()
{
	QAction* action= qobject_cast<QAction*>(sender());
	if (action)
	{
		if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
		addItems(QStringList(action->data().toString()));
		startLoading();
	}
}

// Open a Recent Album
void glc_player::openRecentAlbum()
{
	QAction* action= qobject_cast<QAction*>(sender());
	if (action)
	{
		const QString albumName(action->data().toString());
		qDebug() << albumName;
		openAlbum(albumName);
	}
}

// View and edit mesh property
void glc_player::meshProperty()
{
	EditMeshDialog editMeshDialog(m_OpenglView.context(), m_pCurrentSelection, this);
	editMeshDialog.exec();	
}

// Reframe the view
void glc_player::reframe()
{
	GLC_BoundingBox emptyBoundingBox;
	m_OpenglView.reframe(emptyBoundingBox, true);
	
}
// Select mode
void glc_player::selectMode()
{
	if (!ui.action_Select->isChecked())
	{
		ui.action_Select->setChecked(true);
	}
	ui.action_ViewCenter->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Rotate->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_NORMAL);
}

// define view center
void glc_player::viewCenterMode()
{
	if (!ui.action_ViewCenter->isChecked())
	{
		ui.action_ViewCenter->setChecked(true);
	}
	ui.action_Select->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Rotate->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_POINTING);	
}

// Pan the view
void glc_player::panMode()
{
	if (!ui.action_Pan->isChecked())
	{
		ui.action_Pan->setChecked(true);
	}
	ui.action_Select->setChecked(false);
	ui.action_ViewCenter->setChecked(false);
	ui.action_Rotate->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_PANNING);	
}

// Rotate the view
void glc_player::rotateMode()
{
	if (!ui.action_Rotate->isChecked())
	{
		ui.action_Rotate->setChecked(true);
	}	
	ui.action_Select->setChecked(false);
	ui.action_ViewCenter->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_ORBITING);	
}

// Zoom the view mode
void glc_player::zoomMode()
{
	if (!ui.action_Zoom->isChecked())
	{
		ui.action_Zoom->setChecked(true);
	}	
	ui.action_Select->setChecked(false);
	ui.action_ViewCenter->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Rotate->setChecked(false);
	m_OpenglView.changeEnterState(VE_ZOOMING);	
}

// Show application settings dialog
void glc_player::showSettings()
{
	SettingsDialog settingsDialog(m_DisplayThumbnails, m_iconSize, m_dislayInfoPanel, m_QuitConfirmation, this);
	if (settingsDialog.exec() == QDialog::Accepted)
	{
		bool update= m_DisplayThumbnails != settingsDialog.thumbnailHaveToBeDisplay();
		update= update || (m_iconSize != settingsDialog.getThumbnailSize());
		if (update)
		{
			m_DisplayThumbnails= settingsDialog.thumbnailHaveToBeDisplay();
			// Disable Refresh thumbnail button
			ui.refreshThumbnails->setEnabled(m_DisplayThumbnails & (ui.loadingList->count() != 0));	
			m_iconSize= settingsDialog.getThumbnailSize();
			settingsDialog.hide();
			ui.loadingList->setIconSize(m_iconSize);
			refreshIconInItemList();
		}
		if (m_dislayInfoPanel != settingsDialog.infoHaveToBeShown())
		{
			m_dislayInfoPanel= settingsDialog.infoHaveToBeShown();
			m_OpenglView.setDisplayInfoPanel(m_dislayInfoPanel);
		}
		
		m_QuitConfirmation= settingsDialog.quitConfirmation();
		
	}
	
}
// A file as been Opened
void glc_player::fileOpened()
{
	GLC_World world(m_OpenFileThread.getWorld());
	const QString fileName(m_OpenFileThread.getFileName());
	// Decrement the number of unload file
	--m_NumberOfUnloadedFile;
	// Check if the world as been succesffuly built
	if (!world.isEmpty())
	{
		m_FileEntryHash[fileName].setWorld(world);
		m_FileEntryHash[fileName].setLoadingStatus(false);
		// Update the foreground of the item Opened file
		int curItem= 0;
		while (ui.loadingList->item(curItem)->data(Qt::UserRole).toString() != fileName)
		{
			++curItem;
		}
		ui.loadingList->item(curItem)->setForeground(QBrush(Qt::black));
		
		// Test if the loaded model have to be set as current
		if (m_MakeFirstFileCurrent)
		{
			//Update window Title
			setWindowTitle(QCoreApplication::applicationName() + QString(" [") + QFileInfo(fileName).fileName() + QString("]"));
			// Set loading file as current item
			if (ui.loadingList->item(curItem) != ui.loadingList->currentItem())
			{
				ui.loadingList->setCurrentItem(ui.loadingList->item(curItem));
			}
			else	// force the display of loaded file
			{
				currentFileItemChanged(ui.loadingList->currentItem(), NULL);
			}
		}
		else if (ui.loadingList->item(curItem) == ui.loadingList->currentItem())
		{
			// force the display of loaded file
			currentFileItemChanged(ui.loadingList->currentItem(), NULL);
		}
		// Create the icon of the loaded file
		if (m_DisplayThumbnails)
		{
			computeIconInBackBuffer(curItem);
		}
		
		addToRecentFiles(fileName);
		m_FileLoadingInProgress= false;
		m_MakeFirstFileCurrent= false;
	}
	updateUIState();
	// If there is a other file item, load it
	startLoading();
}
// load of File failed
void glc_player::loadFileFailed()
{
	ui.statusbar->showMessage(tr("File Not Loaded"));
	m_pProgressBar->hide();
	
	// Update the foreground of the item Opened file
	const QString fileName(m_OpenFileThread.getFileName());
	int curItem= 0;
	while (ui.loadingList->item(curItem)->data(Qt::UserRole).toString() != fileName)
	{
		++curItem;
	}
	ui.loadingList->item(curItem)->setForeground(QBrush(Qt::red));
	
	if (m_DisplayThumbnails)
	{
		ui.loadingList->item(curItem)->setIcon(QPixmap(getIconName(true)).scaled(m_iconSize, Qt::IgnoreAspectRatio));
	}
	
	// Update the entry
	m_FileEntryHash[fileName].setLoadingStatus(false);
	m_FileEntryHash[fileName].setError(m_OpenFileThread.getErrorMsg());
	m_FileLoadingInProgress= false;
	// Test if the item on error is the first
	if (m_NumberOfErrorFile == 0)
		ui.removeOnErrorItem->setEnabled(true);
	// increment the number of load failure
	++m_NumberOfErrorFile;
}
// Current file Item Changed
void glc_player::currentFileItemChanged(QListWidgetItem * current, QListWidgetItem * previous )
{
	if ((previous != NULL) && (current != NULL))
	{
		if (NULL != m_pCurrentSelection)
		{
			unselect();
			// view feedback
			//m_OpenglView.unselect();
		}
		QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(previous->data(Qt::UserRole).toString());
		if (iEntry.value().isLoaded() && (!(iEntry.value().getCamera() == m_OpenglView.getCamera())
											|| (iEntry.value().getPolygonMode() != m_OpenglView.getMode())
											|| iEntry.value().getViewAngle() != m_OpenglView.getViewAngle()))
		{
			iEntry.value().setCameraAndAngle(m_OpenglView.getCamera(), m_OpenglView.getViewAngle());
			iEntry.value().setPolygonMode(m_OpenglView.getMode());
			if (m_DisplayThumbnails)
			{
				computeIconInBackBuffer(ui.loadingList->row(previous));
				/*
				QImage snapShoot= m_OpenglView.grabFrameBuffer();
				previous->setIcon(QPixmap::fromImage(snapShoot).scaled(m_iconSize, Qt::IgnoreAspectRatio));
				*/
			}
		}
		if (iEntry.value().errorOccurWhileLoading())
		{
			ui.loadingList->removeAction(m_pActionGetErrorInfo);
		}
	}
	if (current != NULL)
	{
		setCurrentFileItem(current->data(Qt::UserRole).toString());
	}
	else
	{
		if (m_OpenglView.getVisibleState() == false)
		{
			m_OpenglView.setToVisibleState();
		}			
		m_OpenglView.clear();
		m_OpenglView.updateGL();		
	}
	// Update UI
	if (m_OpenglView.isEmpty() && ui.action_IsoView->isEnabled())
	{
		ui.action_IsoView->setEnabled(false);
		ui.action_FrontView->setEnabled(false);
		ui.action_BackView->setEnabled(false);
		ui.action_LeftView->setEnabled(false);
		ui.action_RightView->setEnabled(false);
		ui.action_TopView->setEnabled(false);
		ui.action_BottomView->setEnabled(false);
		ui.action_Reframe->setEnabled(false);
		ui.action_ZoomIn->setEnabled(false);
		ui.action_ZoomOut->setEnabled(false);
		ui.action_RenderPoints->setEnabled(false);
		ui.action_RenderWireframe->setEnabled(false);
		ui.action_RenderShading->setEnabled(false);
		ui.action_SnapShot->setEnabled(false);
		
		ui.actionShow_all->setEnabled(false);
		ui.actionSwap_visible_space->setEnabled(false);
	}
	else if (!m_OpenglView.isEmpty() && !ui.action_IsoView->isEnabled())
	{
		ui.action_IsoView->setEnabled(true);
		ui.action_FrontView->setEnabled(true);
		ui.action_BackView->setEnabled(true);
		ui.action_LeftView->setEnabled(true);
		ui.action_RightView->setEnabled(true);
		ui.action_TopView->setEnabled(true);
		ui.action_BottomView->setEnabled(true);
		ui.action_Reframe->setEnabled(true);
		ui.action_ZoomIn->setEnabled(true);
		ui.action_ZoomOut->setEnabled(true);
		ui.action_RenderPoints->setEnabled(true);
		ui.action_RenderWireframe->setEnabled(true);
		ui.action_RenderShading->setEnabled(true);
		ui.action_SnapShot->setEnabled(true);
		
		ui.actionShow_all->setEnabled(true);
		ui.actionSwap_visible_space->setEnabled(true);

	}
}
// File Item as been clicked
void glc_player::fileItemClicked(QListWidgetItem *pClickedItem)
{
	if (pClickedItem == ui.loadingList->currentItem())
	{
		ui.statusbar->showMessage(pClickedItem->data(Qt::UserRole).toString());
	}
}

// Remove unload item
void glc_player::removeUnloadFileItem()
{
	int currentRow= ui.loadingList->currentRow();
	for (int i= 0; i < m_NumberOfUnloadedFile; ++i)
	{
		int row= ui.loadingList->count() - 1;
		m_FileEntryHash.remove(ui.loadingList->item(row)->data(Qt::UserRole).toString());
		delete ui.loadingList->takeItem(ui.loadingList->count() - 1);
	}
	m_NumberOfUnloadedFile= 0;
	// Update Continu list loading flag
	m_ContinuListLoading= true;
	updateUIState();
	if (currentRow < ui.loadingList->count())
	{
		ui.loadingList->setCurrentRow(currentRow);
	}
	else
	{
		
		ui.loadingList->setCurrentRow(0);
	}
	
	
}
// Remove item on error
void glc_player::removeItemOnError()
{
	QString key;
	QHash<QString, FileEntry>::iterator iEntry;
	int i= 0;
	while ((i < ui.loadingList->count()) && (m_NumberOfErrorFile > 0))
	{
		key= ui.loadingList->item(i)->data(Qt::UserRole).toString();
		iEntry= m_FileEntryHash.find(key);
		if (iEntry.value().errorOccurWhileLoading())
		{
			m_FileEntryHash.remove(key);
			delete ui.loadingList->takeItem(i);
			--m_NumberOfErrorFile;
		}
		else
		{
			++i;
		}
	}
	ui.loadingList->removeAction(m_pActionGetErrorInfo);
	updateUIState();
}

// Start Loading
void glc_player::startLoadingButton()
{
	if (ui.startLoadingButton->isChecked())
	{
		ui.stopLoadingButton->setChecked(false);
		m_ContinuListLoading= true;
		startLoading();
	}
	else
	{
		ui.startLoadingButton->setChecked(true);
	}
	
}
// Stop Loading
void glc_player::stopLoadingButton()
{
	if (ui.stopLoadingButton->isChecked())
	{
		ui.startLoadingButton->setChecked(false);
		m_ContinuListLoading= false;
		QString loadingFileItemName= m_OpenFileThread.getFileName();
		QString msg(QString(tr("Loading will be interrupt after the loading of : ")) + loadingFileItemName);
		QMessageBox::information(this, tr("Loading interruption"), msg);		
	}
	else
	{
		ui.stopLoadingButton->setChecked(true);
	}
}

// Take a snapshoot
void glc_player::takeSnapShot()
{
	if (!m_OpenglView.isEmpty())
	{
		QFileInfo fileInfo(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		QString defaultFileName(fileInfo.absolutePath() + QDir::separator() + fileInfo.baseName() + QString(".png"));		
		QString fileName= QFileDialog::getSaveFileName(this, tr("Save snapShoot"), defaultFileName , tr("Images (*.png *.jpg)"));
		if (!fileName.isEmpty())
		{
			m_OpenglView.setSnapShootMode(true);
			m_OpenglView.setAutoBufferSwap(false);
			m_OpenglView.updateGL();
			QImage snapShoot= m_OpenglView.grabFrameBuffer();
			m_OpenglView.setSnapShootMode(false);
			m_OpenglView.updateGL();
			m_OpenglView.setAutoBufferSwap(true);
			snapShoot.save(fileName);
		}
	}
}
// Delete the currentItem
void glc_player::deleteCurrentItem()
{
	if (NULL != ui.loadingList->currentItem())
	{
		int currentRow= ui.loadingList->currentRow();
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		delete ui.loadingList->takeItem(currentRow);
		// Take the entry from hash table
		FileEntry entry= m_FileEntryHash.take(fileName);
		// Test entry status
		if (entry.errorOccurWhileLoading())
		{
			--m_NumberOfErrorFile;
			ui.loadingList->removeAction(m_pActionGetErrorInfo);
		}
		else if (entry.isReadyToLoad())
		{
			if (--m_NumberOfUnloadedFile == 0)
			{
				// Update Continu list loading flag
				m_ContinuListLoading= true;
			}
		}
		updateUIState();
	}
}

// Manage current item information displaying
void glc_player::manageItemInformationDisplay(int state)
{
	if (state == Qt::Unchecked)
	{
		ui.currentItemGroup->hide();
	}
	else
	{
		ui.currentItemGroup->show();
	}
}
// Album Management visibility toggle
void glc_player::albumManagementVisibilityToggle()
{
	ui.albumManagementWindow->blockSignals(true);
	if (ui.action_AlbumManagement->isChecked())
	{
		ui.albumManagementWindow->show();
	}
	else
	{
		ui.albumManagementWindow->hide();
	}
	ui.albumManagementWindow->blockSignals(false);
}
// The album management visibility change
void glc_player::albumManagementVisibilityChanged(bool isVisible)
{
	ui.action_AlbumManagement->setChecked(isVisible);
}
// Camera property toggle
void glc_player::cameraPropertyVisibilityToggle()
{
	ui.cameraProperty->blockSignals(true);
	if (ui.action_CameraProperty->isChecked())
	{
		ui.cameraProperty->show();
	}
	else
	{
		ui.cameraProperty->hide();
	}
	ui.cameraProperty->blockSignals(false);
	
}
// The camera property visibility change
void glc_player::cameraPropertyVisibilityChanged(bool isVisible)
{
	ui.action_CameraProperty->setChecked(isVisible);
}
// Selection property toggle
void glc_player::selectionPropertyVisibilityToggle()
{
	ui.selectionDockWidget->blockSignals(true);
	if (ui.action_SelectionProperty->isChecked())
	{
		ui.selectionDockWidget->show();
	}
	else
	{
		ui.selectionDockWidget->hide();
	}
	ui.selectionDockWidget->blockSignals(false);
	
}
// The Selection property visibility change
void glc_player::selectionPropertyVisibilityChanged(bool isVisible)
{
	ui.action_SelectionProperty->setChecked(isVisible);
}

// Go to Point Rendering mode
void glc_player::pointsRenderingMode()
{
	if (!ui.action_RenderPoints->isChecked())
	{
		ui.action_RenderPoints->setChecked(true);
	}
	else
	{
		ui.action_RenderWireframe->setChecked(false);
		ui.action_RenderShading->setChecked(false);
		m_OpenglView.setMode(GL_POINT);
		m_OpenglView.updateGL();
	}
}
// Go to wireframe Rendering mode
void glc_player::wireframeRenderingMode()
{
	if (!ui.action_RenderWireframe->isChecked())
	{
		ui.action_RenderWireframe->setChecked(true);
	}
	else
	{
		ui.action_RenderPoints->setChecked(false);
		ui.action_RenderShading->setChecked(false);
		m_OpenglView.setMode(GL_LINE);
		m_OpenglView.updateGL();
	}
}
// Go to shading Rendering Mode
void glc_player::shadingRenderingMode()
{
	if (!ui.action_RenderShading->isChecked())
	{
		ui.action_RenderShading->setChecked(true);
	}
	else
	{
		ui.action_RenderWireframe->setChecked(false);
		ui.action_RenderPoints->setChecked(false);
		m_OpenglView.setMode(GL_FILL);
		m_OpenglView.updateGL();
	}
}
// Display about GLC_Player dialog box
void glc_player::aboutPlayer()
{
	AboutPlayer about(this);
	about.exec();
}

// Go to fullScreen Mode
void glc_player::fullScreen()
{
	// Toggle fullScreen Mode
	setWindowState(windowState() ^ Qt::WindowFullScreen);	
}
// Refresh the icon in item list
void glc_player::refreshIconInItemList()
{
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	// The number of item in the list
	const int max= ui.loadingList->count();
	// An iterator to the FileEntry HashTable
	QHash<QString, FileEntry>::iterator iEntry;	
	if (m_DisplayThumbnails)
	{		
		// Show the progress bar
		ui.statusbar->showMessage(tr("Creating Thumbnails Please Wait..."));
		m_pProgressBar->show();
		m_pProgressBar->setValue(0);			

		// Thumbnail have to be displayed
		ui.loadingList->setIconSize(m_iconSize);

		// Get the icon name
		for (int curListItem= 0; curListItem < max; ++curListItem)
		{
			iEntry= m_FileEntryHash.find(ui.loadingList->item(curListItem)->data(Qt::UserRole).toString());
			if (iEntry.value().isLoaded())
			{
				computeIconInBackBuffer(curListItem);
			}
			else if (iEntry.value().getError().isEmpty())
			{				
				ui.loadingList->item(curListItem)->setIcon(QPixmap(getIconName(false)).scaled(m_iconSize, Qt::IgnoreAspectRatio));
			}
			else
			{				
				ui.loadingList->item(curListItem)->setIcon(QPixmap(getIconName(true)).scaled(m_iconSize, Qt::IgnoreAspectRatio));
			}
			
			m_pProgressBar->setValue(static_cast<int>(static_cast<double>((curListItem + 1)) * (100.0 / max)));
		}
		
		ui.statusbar->showMessage(tr("Thumbnails successfully created"));
		// Force items to be laid out again
		ui.loadingList->setSpacing(1);
		m_pProgressBar->hide();
	}
	else
	{
		// Thumbnail haven't to be displayed use the default one
		ui.loadingList->setIconSize(QSize());
		for (int curListItem= 0; curListItem < max; ++curListItem)
		{
			ui.loadingList->item(curListItem)->setIcon(QIcon());
		}
	}	QApplication::restoreOverrideCursor();
	
}
// Display GLC_Player help page
void glc_player::help()
{
	QSettings settings;
	if ("fr" == settings.value("currentLanguage").toString())
	{
		QDesktopServices::openUrl(QUrl("http://glc-player.net/fr/doc.php"));
	}
	else
	{
		QDesktopServices::openUrl(QUrl("http://glc-player.net/doc.php"));
	}
}

// Reverse current model normal
void glc_player::reverseModelNormal()
{
	if (NULL != ui.loadingList->currentItem())
	{
		QHash<QString, FileEntry>::iterator iEntry;
		iEntry= m_FileEntryHash.find(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		iEntry.value().getWorld().reversePartNormal();
		m_OpenglView.updateGL();
	}
}
// Update the view
void glc_player::updateView()
{
	m_OpenglView.setDistMinAndMax();
	m_OpenglView.updateGL();
}

// Get information about failed loading
void glc_player::getErrorInformation()
{
	if (NULL != ui.loadingList->currentItem())
	{
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		// Take the entry from hash table
		QString message(m_FileEntryHash.value(fileName).getError());
		QMessageBox::information(this, QCoreApplication::applicationName(), message);
	}
}

// Edit the light properties
void glc_player::editLightDialog()
{
	if (m_pEditLightDialog == NULL)
	{
		m_pEditLightDialog= new EditLightDialog(m_OpenglView.getLight(), this);
		connect(m_pEditLightDialog, SIGNAL(lightUpdated()), &m_OpenglView, SLOT(updateGL()));
	}
	m_pEditLightDialog->setWindowFlags(Qt::Tool);
	m_pEditLightDialog->show();
	m_pEditLightDialog->raise();
	m_pEditLightDialog->activateWindow();
}
// Two sided lightning
void glc_player::twoSidedLightning()
{
		m_OpenglView.getLight()->setTwoSided(ui.actionTwo_sided_Lightning->isChecked());
}

//////////////////////////////////////////////////////////////////////
// Private services Functions
//////////////////////////////////////////////////////////////////////

// Create RecentFileActionArray
void glc_player::createRecentFileActionsArray()
{
	for (int i= 0; i < MaxRecentFiles; ++i)
	{
		m_pRecentFileActionsArray[i]= new QAction(this);
		m_pRecentFileActionsArray[i]->setVisible(false);
		ui.menuRecent_Models->addAction(m_pRecentFileActionsArray[i]);
		connect(m_pRecentFileActionsArray[i], SIGNAL(triggered()), this, SLOT(openRecentFile()));
	}
}

// Create RecentFileActionArray
void glc_player::createRecentAlbumActionsArray()
{
	for (int i= 0; i < MaxRecentFiles; ++i)
	{
		m_pRecentAlbumActionsArray[i]= new QAction(this);
		m_pRecentAlbumActionsArray[i]->setVisible(false);
		ui.menuRecent_Album->addAction(m_pRecentAlbumActionsArray[i]);
		connect(m_pRecentAlbumActionsArray[i], SIGNAL(triggered()), this, SLOT(openRecentAlbum()));
	}
}

// Update Recent Files
void glc_player::updateRecentsFiles()
{
	QMutableStringListIterator i(m_RecentFilesList);
	while (i.hasNext())
	{
		if (!QFile::exists(i.next()))
		{
			i.remove();
		}
	}
	for (int j= 0; j < MaxRecentFiles;  ++j)
	{
		if (j < m_RecentFilesList.count())
		{
			QString text= tr("&%1 %2").arg(j + 1).arg(QFileInfo(m_RecentFilesList[j]).fileName());
			m_pRecentFileActionsArray[j]->setText(text);
			m_pRecentFileActionsArray[j]->setData(m_RecentFilesList[j]);
			m_pRecentFileActionsArray[j]->setVisible(true);
		}
		else
		{
			m_pRecentFileActionsArray[j]->setVisible(false);
		}
	}
}

// Update Recent Files
void glc_player::updateRecentsAlbums()
{
	QMutableStringListIterator i(m_RecentAlbumsList);
	while (i.hasNext())
	{
		if (!QFile::exists(i.next()))
		{
			i.remove();
		}
	}
	for (int j= 0; j < MaxRecentFiles;  ++j)
	{
		if (j < m_RecentAlbumsList.count())
		{
			QString text= tr("&%1 %2").arg(j + 1).arg(QFileInfo(m_RecentAlbumsList[j]).fileName());
			m_pRecentAlbumActionsArray[j]->setText(text);
			m_pRecentAlbumActionsArray[j]->setData(m_RecentAlbumsList[j]);
			m_pRecentAlbumActionsArray[j]->setVisible(true);
		}
		else
		{
			m_pRecentAlbumActionsArray[j]->setVisible(false);
		}
	}
}

// Open the file
void glc_player::openFile(const QString& fileName)
{
	QFile file(fileName);
	m_FileLoadingInProgress= true;
	m_CurrentPath= QFileInfo(fileName).absolutePath();
	m_pProgressBar->setValue(0);
	m_pProgressBar->show();
	m_OpenFileThread.setOpenFile(&file);
	m_OpenFileThread.start(QThread::LowPriority);	
	ui.startLoadingButton->setEnabled(false);
	ui.action_NewAlbum->setEnabled(false);
	ui.action_OpenAlbum->setEnabled(false);
	ui.menuRecent_Album->setEnabled(false);
	ui.action_SaveAlbumAs->setEnabled(false);
	ui.action_SaveAlbum->setEnabled(false);
	ui.refreshThumbnails->setEnabled(false);
	ui.removeUnloadItemButton->setEnabled(false);
	if (m_NumberOfUnloadedFile > 1)
	{
		ui.stopLoadingButton->setEnabled(true);
	}
	else
	{
		ui.stopLoadingButton->setEnabled(false);
	}		
}

// Add current file to recent file
void glc_player::addToRecentFiles(const QString& fileName)
{
	m_CurrentFileName= fileName;
	if (!m_CurrentFileName.isEmpty())
	{
		m_RecentFilesList.removeAll(m_CurrentFileName);
		m_RecentFilesList.prepend(m_CurrentFileName);
		if (m_RecentFilesList.size() > MaxRecentFiles)
		{
			while (m_RecentFilesList.size() > MaxRecentFiles)
			{
				m_RecentFilesList.removeLast();
			}
		}
		updateRecentsFiles();
	}
}

// Add current Album to recent Albums
void glc_player::addToRecentAlbums(const QString& fileName)
{
	m_CurrentAlbumName= fileName;
	if (!m_CurrentAlbumName.isEmpty())
	{
		m_RecentAlbumsList.removeAll(m_CurrentAlbumName);
		m_RecentAlbumsList.prepend(m_CurrentAlbumName);
		if (m_RecentAlbumsList.size() > MaxRecentFiles)
		{
			while (m_RecentAlbumsList.size() > MaxRecentFiles)
			{
				m_RecentAlbumsList.removeLast();
			}
		}
		updateRecentsAlbums();
	}
}

// drag enter event
void glc_player::dragEnterEvent(QDragEnterEvent* event)
{
	if (event->mimeData()->hasFormat("text/uri-list"))
	{
		event->acceptProposedAction();
	}
}

// drop event
void glc_player::dropEvent(QDropEvent* event)
{
	QList<QUrl> urls= event->mimeData()->urls();
	if (urls.isEmpty()) return;
	// Stransform the url List to string list
	QStringList localFiles;
	for (int i= 0; i < urls.size(); ++i)
	{
		localFiles << urls[i].toLocalFile();
	}
	if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
	const bool fileIsAlbum= (QFileInfo(localFiles[0]).suffix().toLower() == "album");
	if (fileIsAlbum and ui.action_OpenAlbum->isEnabled())
	{
		openAlbum(localFiles[0]);
	}
	else if (not fileIsAlbum)
	{
		addItems(localFiles);
		startLoading();
	}
}

// Load successful
void glc_player::loadSuccessful(QString fileName)
{
	addToRecentFiles(fileName);
	m_FileLoadingInProgress= false;
}

// Add items
void glc_player::addItems(QStringList nameList)
{
	QString currentString;
	QString iconName= getIconName(false);
	for (int i= 0; i < nameList.size(); ++i)
	{
		// If the file is already in the list the file is skipped
		if (!m_FileEntryHash.contains(nameList[i]) && 
				((QFileInfo(nameList[i]).suffix().toLower() == "obj") || (QFileInfo(nameList[i]).suffix().toLower() == "stl")
						|| (QFileInfo(nameList[i]).suffix().toLower() == "off") || (QFileInfo(nameList[i]).suffix().toLower() == "3ds")))
		{
			++m_NumberOfUnloadedFile;
			QFileInfo fileInfo(nameList[i]);
			QListWidgetItem* pItem= new QListWidgetItem(fileInfo.fileName());
			pItem->setData(Qt::UserRole, QVariant(nameList[i]));
			pItem->setForeground(QBrush(Qt::gray));
			if (m_DisplayThumbnails)
			{
				pItem->setIcon(QPixmap(iconName).scaled(m_iconSize, Qt::IgnoreAspectRatio));
			}
			m_FileEntryHash.insert(nameList[i], FileEntry(nameList[i]));
			ui.loadingList->addItem(pItem);
		}
	}
	updateUIState();
}
// Start Loading FileItem
void glc_player::startLoading()
{
	// Test if application is loading a file
	if (m_FileLoadingInProgress) return;
	
	int curListItem= 0;
	const int max= ui.loadingList->count();
	
	QHash<QString, FileEntry>::iterator iEntry;
	// Test if the list item is empty
	if (curListItem >= max)
	{
		return;
	}
	iEntry= m_FileEntryHash.find(ui.loadingList->item(curListItem)->data(Qt::UserRole).toString());
	while (curListItem < max && !iEntry.value().isReadyToLoad() && m_ContinuListLoading)
	{
		++curListItem;
		if ((curListItem < max))
		{
			iEntry= m_FileEntryHash.find(ui.loadingList->item(curListItem)->data(Qt::UserRole).toString());
		}
	}
	
	// test if the Entry have been find and continu loading == true
	if ((curListItem < max) && m_ContinuListLoading)
	{
		iEntry.value().setLoadingStatus(true);
		openFile(iEntry.value().getFileName());
		m_ListLoadingInProgress= true;
	}
	else if ((curListItem >= max))
	{
		m_ListLoadingInProgress= false;
		ui.startLoadingButton->setEnabled(false);
		ui.stopLoadingButton->setEnabled(false);
		ui.action_NewAlbum->setEnabled(true);
		ui.action_OpenAlbum->setEnabled(true);
		ui.menuRecent_Album->setEnabled(true);
		ui.action_SaveAlbumAs->setEnabled(true);
		ui.action_SaveAlbum->setEnabled(true);
		ui.refreshThumbnails->setEnabled(m_DisplayThumbnails);
	}
	else
	{
		ui.startLoadingButton->setEnabled(true);
		ui.stopLoadingButton->setEnabled(false);
		ui.action_NewAlbum->setEnabled(true);
		ui.action_OpenAlbum->setEnabled(true);
		ui.menuRecent_Album->setEnabled(true);
		ui.action_SaveAlbumAs->setEnabled(true);
		ui.action_SaveAlbum->setEnabled(true);
		ui.refreshThumbnails->setEnabled(m_DisplayThumbnails);
		ui.removeUnloadItemButton->setEnabled(true);
	}		
}
// Set the current File Item
void glc_player::setCurrentFileItem(const QString& fileName)
{
	QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(fileName);
	// Check if the get error information has to be added to contextual menu
	if (iEntry.value().errorOccurWhileLoading())
	{
		ui.loadingList->addAction(m_pActionGetErrorInfo);
		m_pActionDeleteSelectedItem->setEnabled(true);
	}
	if ((iEntry != m_FileEntryHash.constEnd()) && (iEntry.value().isLoaded()))
	{
		// Set polygon mode of the entry
		iEntry.value().setPolygonMode(m_OpenglView.getMode());
		GLC_World world(iEntry.value().getWorld());
		// Check the visible state of the view
		if (m_OpenglView.getVisibleState() != world.collection()->getShowState())
		{
			if(m_OpenglView.getVisibleState() == true) m_OpenglView.setToInVisibleState();
			else m_OpenglView.setToVisibleState();
		}
		m_OpenglView.clear();
		m_OpenglView.add(world);
		if (iEntry.value().cameraIsSet())
		{
			m_OpenglView.setCameraAndAngle(iEntry.value().getCamera(), iEntry.value().getViewAngle());
			m_OpenglView.updateGL();
		}
		else
		{
			// Define iso position of the camera eye
			GLC_Vector4d VectEye(1.0, 1.0, 1.0);
			VectEye= VectEye + GLC_Vector4d();
			// Up vector
			const GLC_Vector4d VectUp(0.0, 1.0, 0.0);
			m_OpenglView.setCameraAndAngle(GLC_Camera(VectEye, GLC_Vector4d(), VectUp), iEntry.value().getViewAngle());
			
			m_OpenglView.reframe(GLC_BoundingBox(), false);
		}
		setWindowTitle(QString(QCoreApplication::applicationName() +" [") + QFileInfo(fileName).fileName() + QString("]"));
		ui.statusbar->showMessage(fileName);
		ui.fileSize->setText(QString::number(QFileInfo(fileName).size() / 1000));
		ui.numberOfMesh->setText(QString::number(iEntry.value().getNumberOfInstance()));
		ui.numberOfFaces->setText(QString::number(iEntry.value().getNumberOfFaces()));
		ui.numberOfVertex->setText(QString::number(iEntry.value().getNumberOfVertex()));
		ui.reverseNormals->setEnabled(true);
		// Enable the possibility to remove current item
		m_pActionDeleteSelectedItem->setEnabled(true);
		// Manage selection
		if (!world.collection()->getSelections()->empty())
		{
			m_pCurrentSelection= world.collection()->getSelections()->begin().value();
			m_pSelectionProperty->setSelection(m_pCurrentSelection);
			ui.action_Property->setEnabled(true);
			ui.actionUnselect->setEnabled(true);
			ui.actionShow_Hide->setEnabled(true);
			ui.actionHide_unselected->setEnabled(true);
		}

	}
	else // Entry not found or not loaded => clear the view
	{
		if (iEntry.value().isLoading())
		{
			// Disable the possibility to remove current item
			m_pActionDeleteSelectedItem->setEnabled(false);					
		}
		ui.statusbar->showMessage(fileName);
		ui.fileSize->setText(QString::number(QFileInfo(fileName).size() / 1000));
		ui.numberOfMesh->clear();
		ui.numberOfFaces->clear();
		ui.numberOfVertex->clear();
		ui.reverseNormals->setEnabled(false);
		if (m_OpenglView.getVisibleState() == false)
		{
			m_OpenglView.setToVisibleState();
		}
		m_OpenglView.clear();
		m_OpenglView.updateGL();
	}
}
// Update the UI State
void glc_player::updateUIState()
{
	if (ui.loadingList->count() == 0)
	{
		// restore default album name
		ui.labelAlbumName->setText(tr("New Album"));
		// Disable New album action
		ui.action_NewAlbum->setEnabled(false);
		// Disable Save Album As
		ui.action_SaveAlbumAs->setEnabled(false);
		ui.action_SaveAlbum->setEnabled(false);
		// Disable Refresh thumbnail button
		ui.refreshThumbnails->setEnabled(false);
		// Disable all loading list action
		ui.startLoadingButton->setEnabled(false);
		ui.stopLoadingButton->setEnabled(false);
		ui.removeUnloadItemButton->setEnabled(false);
		ui.removeOnErrorItem->setEnabled(false);
		// Uncheck start and stop button
		ui.stopLoadingButton->setChecked(false);
		ui.startLoadingButton->setChecked(false);
		//Update window Title
		setWindowTitle(QCoreApplication::applicationName());
	
		ui.statusbar->showMessage(tr("Untiteled"));
		ui.fileSize->clear();
		ui.numberOfMesh->clear();
		ui.numberOfFaces->clear();
		ui.numberOfVertex->clear();
		ui.reverseNormals->setEnabled(false);
		// Disable the possibility to remove current item
		m_pActionDeleteSelectedItem->setEnabled(false);		
		
	}else if (0 == m_NumberOfUnloadedFile)
	{
		// Disable action related action
		ui.startLoadingButton->setEnabled(false);
		ui.stopLoadingButton->setEnabled(false);
		ui.removeUnloadItemButton->setEnabled(false);
		// Uncheck start and stop buttons
		ui.stopLoadingButton->setChecked(false);
		ui.startLoadingButton->setChecked(false);
	}
	ui.numberOfItem->setText(QString::number(ui.loadingList->count()));
	ui.numberOfLoadedItem->setText(QString::number(ui.loadingList->count() - m_NumberOfUnloadedFile));
	ui.removeOnErrorItem->setEnabled(m_NumberOfErrorFile != 0);
}

// Compute the icon of a newly loaded file in back buffer
void glc_player::computeIconInBackBuffer(int itemRow)
{
	GLC_Camera savCam;
	double savAngle;
	bool itemIsNotCurrent= true;
	const int currentRow= ui.loadingList->currentRow();
	bool viewIsEmpty= m_OpenglView.isEmpty();
	m_OpenglView.setSnapShootMode(true);
	// Test if the current item is already in the current view and loaded
	if ((itemRow != currentRow) || (itemRow == currentRow && viewIsEmpty))
	{
		/* The Current model is not in current view
		 * Or
		 * The Current model is in current view but not already in the view
		 */
		QString fileName(ui.loadingList->item(itemRow)->data(Qt::UserRole).toString());
		
		QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(fileName);
		// Test if the model is loaded
		if (iEntry.value().isLoaded())
		{
			// The model is Loaded
			savCam= m_OpenglView.getCamera();
			savAngle= m_OpenglView.getViewAngle();
			// Set polygon mode of the entry
			iEntry.value().setPolygonMode(m_OpenglView.getMode());
			GLC_World world(iEntry.value().getWorld());
			m_OpenglView.setAutoBufferSwap(false);
			if (m_OpenglView.getVisibleState() != world.collection()->getShowState())
			{
				if(m_OpenglView.getVisibleState() == true) m_OpenglView.setToInVisibleState();
				else m_OpenglView.setToVisibleState();
			}

			m_OpenglView.clear();
			m_OpenglView.add(world);
			// Test if the current model's camera is already set
			if (iEntry.value().cameraIsSet())
			{
				// The model's camera is set
				m_OpenglView.setCameraAndAngle(iEntry.value().getCamera(), iEntry.value().getViewAngle());
				m_OpenglView.updateGL();
			}
			else
			{
				// The model's camera is not set
				GLC_BoundingBox emptyBoundingBox;
				m_OpenglView.isoView(false);
				m_OpenglView.reframe(emptyBoundingBox, false); // an updateGL is done while reframing
				iEntry.value().setCameraAndAngle(m_OpenglView.getCamera(), m_OpenglView.getViewAngle());
			}			
		}
		
	}
	else
	{
		/* The Current model is in current view
		 * And
		 * The Current model is already in the view
		 */
		m_OpenglView.setAutoBufferSwap(false);
		m_OpenglView.updateGL();
		itemIsNotCurrent= false;
	}
	// Make the screenShoot
	QImage snapShoot= m_OpenglView.grabFrameBuffer();
	ui.loadingList->item(itemRow)->setIcon(QPixmap::fromImage(snapShoot).scaled(m_iconSize, Qt::IgnoreAspectRatio));
	
	// Test if this is the current model which has been grab
	if (itemIsNotCurrent)
	{
		/* This is not the current model which has been grab
		 * Load The current model in the view
		 */
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		
		QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(fileName);
		// Test if the current model has been loaded
		if (iEntry.value().isLoaded())
		{
			// The current model has been loaded
			// Set polygon mode of the entry
			iEntry.value().setPolygonMode(m_OpenglView.getMode());
			GLC_World world(iEntry.value().getWorld());
			if (m_OpenglView.getVisibleState() != world.collection()->getShowState())
			{
				if(m_OpenglView.getVisibleState() == true) m_OpenglView.setToInVisibleState();
				else m_OpenglView.setToVisibleState();
			}
			m_OpenglView.clear();
			m_OpenglView.add(world);
			m_OpenglView.setCameraAndAngle(savCam, savAngle);
		}
		else
		{
			// The current model has not been loaded
			m_OpenglView.clear();
		}
	}
	m_OpenglView.setAutoBufferSwap(true);
	m_OpenglView.setSnapShootMode(false);
	//m_OpenglView.updateGL();
}

// Return icon name with a size
QString glc_player::getIconName(bool WithError) const
{
	// Built the icon name
	QString iconName;
	if (WithError)
	{
		iconName= ":/images/LogoError";
	}
	else
	{
		iconName= ":/images/Logo";
	}
	
	if (m_iconSize.width() == 32)
	{
		iconName.append(".png");
	}
	else if (m_iconSize.width() == 40)
	{
		iconName.append("40x40.png");
	}	
	else if (m_iconSize.width() == 50)
	{
		iconName.append("50x50.png");
	}
	else if (m_iconSize.width() == 60)
	{
		iconName.append("60x60.png");
	}
	else if (m_iconSize.width() == 70)
	{
		iconName.append("70x70.png");
	}	
	else if (m_iconSize.width() == 80)
	{
		iconName.append("80x80.png");
	}
	return iconName;
}

// Read Application Settings
void glc_player::readSettings()
{
	QSettings settings;
	// Recent Model list
	m_RecentFilesList= settings.value("recentFiles").toStringList();	
	// Recent Album list
	m_RecentAlbumsList= settings.value("recentAlbums").toStringList();	
	// Quit confirmation
	m_QuitConfirmation= settings.value("quitConfirmation", true).toBool();
	
	// MainWindow geometry
	settings.beginGroup("MainWindow");
	resize(settings.value("size", QSize(1200, 730)).toSize());
	move(settings.value("pos", QPoint(50, 50)).toPoint());
	m_dislayInfoPanel= settings.value("displayInfoPanel", true).toBool();
	settings.endGroup();
	
	
	// Album Manager
	settings.beginGroup("AlbumManager");
	// Album manager visibility
	bool visibility= settings.value("visibility", true).toBool();
	ui.action_AlbumManagement->setChecked(visibility);
	ui.albumManagementWindow->setVisible(visibility);
	// Item Information visibility
	visibility= settings.value("itemInformationVisibility", false).toBool();
	ui.displayItemCheckBox->setChecked(visibility);
	ui.currentItemGroup->setVisible(visibility);
	// IconSize
	m_iconSize= settings.value("thumbnailSize", QSize(40, 40)).toSize();
	// Thumbnails display
	m_DisplayThumbnails= settings.value("displayThumbnail", true).toBool();
	settings.endGroup();
	
	// Camera properties
	settings.beginGroup("CameraProperties");
	// Camera properties visibility
	visibility= settings.value("visibility", true).toBool();
	ui.action_CameraProperty->setChecked(visibility);
	ui.cameraProperty->setVisible(visibility);
	settings.endGroup();
	
	// Selection properties
	settings.beginGroup("SelectionProperties");
	// Camera properties visibility
	visibility= settings.value("visibility", true).toBool();
	ui.action_SelectionProperty->setChecked(visibility);
	ui.selectionDockWidget->setVisible(visibility);
	settings.endGroup();

}

// Write Application Settings
void glc_player::writeSettings()
{
	QSettings settings;
	
	// Recent Model list
	settings.setValue("recentFiles", m_RecentFilesList);
	// Recent Albums list
	settings.setValue("recentAlbums", m_RecentAlbumsList);
	// Quit Confirmation
	settings.setValue("quitConfirmation", m_QuitConfirmation);
	
	// MainWindow geometry
	if (isFullScreen())
	{
		// Toggle fullScreen Mode
		setWindowState(windowState() ^ Qt::WindowFullScreen);			
	}
	// MainWindow geometry
	settings.beginGroup("MainWindow");	
	settings.setValue("size", size());
	settings.setValue("pos", pos());
	settings.setValue("displayInfoPanel", m_dislayInfoPanel);
	settings.endGroup();

	// Album Manager
	settings.beginGroup("AlbumManager");
	// Album manager visibility
	settings.setValue("visibility", ui.albumManagementWindow->isVisible());
	// Item Information visibility
	settings.setValue("itemInformationVisibility", ui.currentItemGroup->isVisible());
	// IconSize
	settings.setValue("thumbnailSize", m_iconSize);
	// Thumbnails display
	settings.setValue("displayThumbnail", m_DisplayThumbnails);
	settings.endGroup();
	
	// Camera properties
	settings.beginGroup("CameraProperties");
	// Camera properties visibility
	settings.setValue("visibility", ui.cameraProperty->isVisible());
	settings.endGroup();
	
	// Selection properties
	settings.beginGroup("SelectionProperties");
	// Camera properties visibility
	settings.setValue("visibility", ui.selectionDockWidget->isVisible());
	settings.endGroup();

}

// Ask for saving the current Album
bool glc_player::saveCurrentAlbumDlg()
{
	int ret= QMessageBox::question(this, tr("Save Album"),
							 tr("Save current album before opening the new one?"), QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
	if (ret == QMessageBox::Yes)
	{
		saveAlbum();
		return true;
	}
	else if (ret == QMessageBox::Cancel) return false;
	else return true;

}
// Apply album saving
void glc_player::applySavingAlbum(const QString& fileName)
{
	// Update the current FileEntry camera position
	QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
	if (iEntry.value().isLoaded() && (!(iEntry.value().getCamera() == m_OpenglView.getCamera())
						|| (iEntry.value().getPolygonMode() != m_OpenglView.getMode())
						|| iEntry.value().getViewAngle() != m_OpenglView.getViewAngle()))
	{
		iEntry.value().setCameraAndAngle(m_OpenglView.getCamera(), m_OpenglView.getViewAngle());
		iEntry.value().setPolygonMode(m_OpenglView.getMode());
	}
	
	// Save the album File
	QFile albumFile(fileName);
	AlbumFile albumFileWriter;
	albumFileWriter.saveAlbumFile(m_FileEntryHash.values(), &albumFile);
	m_CurrentAlbumPath= QFileInfo(fileName).absolutePath();
	m_CurrentAlbumName= fileName;
	addToRecentAlbums(fileName);
	QString message(QString(tr("Album (")) + fileName + QString(tr(") successfully saved")));
	ui.statusbar->showMessage(message);
	ui.labelAlbumName->setText(QString("Album : ") + QFileInfo(fileName).baseName());

}

// Open specified album
void glc_player::openAlbum(const QString& fileName)
{
	// Open the album File
	// List of opened models
	QFile albumFile(fileName);
	if (!albumFile.exists())
	{
		qDebug() << "Album File doesn't exist" << fileName;
		return;
	}
	// Test if there is some models in the current album
	if (m_FileEntryHash.size() > 0)
	{
		if (!saveCurrentAlbumDlg()) return; // Do nothing on cancel
		newAlbum(false);
	}
	m_CurrentAlbumPath= QFileInfo(fileName).absolutePath();
	m_CurrentAlbumName= fileName;
	AlbumFile albumFileReader;
	QList<FileEntry> fileEntries;
	try
	{
		fileEntries= albumFileReader.loadAlbumFile(&albumFile);
	}
	catch (GLC_Exception &e)
	{
		QString message(tr("Wrong album file format"));
		QMessageBox::critical(this, QCoreApplication::applicationName(), message);
		return;
	}
	const int max= fileEntries.size();
	QString iconName= getIconName(false);
	if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
	for (int i= 0; i < max; ++i)
	{
		// If the Entry is already in the list the Entry is skipped
		QString fileName(fileEntries[i].getFileName());
		if (!m_FileEntryHash.contains(fileName))
		{
			++m_NumberOfUnloadedFile;
			QFileInfo fileInfo(fileName);
			QListWidgetItem* pItem= new QListWidgetItem(fileInfo.fileName());
			pItem->setData(Qt::UserRole, QVariant(fileName));
			pItem->setForeground(QBrush(Qt::gray));
			if (m_DisplayThumbnails)
			{
				pItem->setIcon(QPixmap(iconName).scaled(m_iconSize, Qt::IgnoreAspectRatio));
			}
			m_FileEntryHash.insert(fileName, fileEntries[i]);
			ui.loadingList->addItem(pItem);
		}				
	}
	addToRecentAlbums(m_CurrentAlbumName);
	ui.labelAlbumName->setText(QString("Album : ") + QFileInfo(fileName).baseName());
	updateUIState();
	startLoading();		
	
}
